#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2016 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Block with the @block decorator function. """
import inspect

# from functools import wraps
import functools

import myhdl
from myhdl import BlockError, BlockInstanceError, Cosimulation
from myhdl._instance import _Instantiator
from myhdl._util import _flatten
from myhdl._extractHierarchy import (_makeMemInfo,
                                     _UserVerilogCode, _UserVhdlCode,
                                     _UserVerilogInstance, _UserVhdlInstance)
from myhdl._Signal import _Signal, _isListOfSigs
from myhdl._misc import isboundmethod

from weakref import WeakValueDictionary


class _error:
    pass


_error.ArgType = "%s: A block should return block or instantiator objects"
_error.InstanceError = "%s: subblock %s should be encapsulated in a block decorator"


class _CallInfo(object):

    def __init__(self, name, modctxt, symdict):
        self.name = name
        self.modctxt = modctxt
        self.symdict = symdict


def _getCallInfo():
    """Get info on the caller of a BlockInstance.

    A BlockInstance should be used in a block context.
    This function gets the required info from the caller
    It uses the frame stack:
    0: this function
    1: block instance constructor
    2: the decorator function call
    3: the function that defines instances
    4: the caller of the block function, e.g. a BlockInstance.

    """

    stack = inspect.stack()
    # caller may be undefined if instantiation from a Python module
    callerrec = None
    funcrec = stack[3]
    name = funcrec[3]
    if len(stack) > 4:
        callerrec = stack[4]
    # special case for list comprehension's extra scope in PY3
    if name == '<listcomp>':
        funcrec = stack[4]
        if len(stack) > 5:
            callerrec = stack[5]

    name = funcrec[3]
    frame = funcrec[0]
    symdict = dict(frame.f_globals)
    symdict.update(frame.f_locals)
    modctxt = False
    if callerrec is not None:
        f_locals = callerrec[0].f_locals
        if 'self' in f_locals:
            modctxt = isinstance(f_locals['self'], _Block)

    return _CallInfo(name, modctxt, symdict)


# ## I don't think this is the right place for uniqueifying the name.
# ## This seems to me to be a conversion concern, not a block concern, and
# ## there should not be the corresponding global state to be maintained here.
# ## The name should be whatever it is, which is then uniqueified at
# ## conversion time. Perhaps this happens already (FIXME - check and fix)
# ## ~ H Gomersall 24/11/2017
_inst_name_set = set()
_name_set = set()


def _uniqueify_name(proposed_name):
    '''
        Creates a unique block name from the proposed name by appending
        a suitable number to the end. Every name this function returns is
        assumed to be used, so will not be returned again.
    '''
    n = 0

    while proposed_name in _name_set:
        proposed_name = proposed_name + '_' + str(n)
        n += 1

    _name_set.add(proposed_name)

    return proposed_name


class _bound_function_wrapper(object):

    def __init__(self, bound_func, srcfile, srcline, skipname):
        self.srcfile = srcfile
        self.srcline = srcline
        self.skipname = skipname
        self.bound_func = bound_func
        functools.update_wrapper(self, bound_func)
        self.calls = 0
        # register the block
        myhdl._simulator._blocks.append(self)

        self.name_prefix = None
        self.name = None

    def __call__(self, *args, **kwargs):
        if self.skipname:
            name = None
        else:
            # name = self.name_prefix + '_' + self.bound_func.__name__ +  str(self.calls)
            name = f'{self.name_prefix}_{self.bound_func.__name__}{self.calls}'
            self.calls += 1
            # See concerns above about uniqueifying
            name = _uniqueify_name(name)

        return _Block(self.bound_func, self, name, self.srcfile,
                      self.srcline, self.skipname, *args, **kwargs)


class block_decorator(object):
    '''
        this code is borrowed (and slightly modified) from J. Villar's PR #328
        https://github.com/myhdl/myhdl/pull/328
        as this was a PR from 2020 it had some issues to be merged, especially the 
        missing sub-version number
    '''
    # TODO: revisit this code and check `self.name` ...
    skipname = False
    ident_method = "get_instance_ident"

    def __init__(self, func):
        self.srcfile = inspect.getsourcefile(func)
        self.srcline = inspect.getsourcelines(func)[0]
        self.func = func
        functools.update_wrapper(self, func)
        self.calls = 0
        self.name = func.__name__

        # register the block
        myhdl._simulator._blocks.append(self)

        self.bound_functions = WeakValueDictionary()

    @classmethod
    def set_decorator_parameters(cls, **kwargs):
        for param_name, value in kwargs.items:
            setattr(cls, param_name, value)

    def __get__(self, instance, owner):
        bound_key = (id(instance), id(owner))

        if bound_key not in self.bound_functions:
            bound_func = self.func.__get__(instance, owner)
            function_wrapper = _bound_function_wrapper(bound_func, self.srcfile, self.srcline, self.skipname)
            self.bound_functions[bound_key] = function_wrapper

            proposed_inst_name = owner.__name__ + '0'

            if self.skipname:
                function_wrapper.name_prefix = None
            else:
                if hasattr(instance, self.ident_method) and callable(getattr(instance, self.ident_method)):
                    proposed_inst_name = getattr(instance, self.ident_method)()
                else:
                    proposed_inst_name = owner.__name__ + '0'
                    n = 1
                    while proposed_inst_name in _inst_name_set:
                        proposed_inst_name = owner.__name__ + str(n)
                        n += 1

                function_wrapper.name_prefix = proposed_inst_name
                _inst_name_set.add(proposed_inst_name)
            self.name = proposed_inst_name

        else:
            function_wrapper = self.bound_functions[bound_key]
            bound_func = self.bound_functions[bound_key]

        return function_wrapper

    def __call__(self, *args, **kwargs):
        if self.skipname:
            name = None
        else:
            name = self.func.__name__ + str(self.calls)
            self.calls += 1
            # See concerns above about uniqueifying
            name = _uniqueify_name(name)

        return _Block(self.func, self, name, self.srcfile,
                      self.srcline, self.skipname, *args, **kwargs)


def block(func=None, **kwargs):
    decorator = block_decorator
    if func is None:
        return type(decorator.__name__, (decorator,), kwargs)
    else:
        return decorator(func)


class _Block(object):

    def __init__(self, func, deco, name, srcfile, srcline, skipname, *args, **kwargs):
        calls = deco.calls

        self.func = func
        self.args = args
        self.kwargs = kwargs
        self.__doc__ = func.__doc__
        callinfo = _getCallInfo()
        self.callinfo = callinfo
        self.modctxt = callinfo.modctxt
        self.callername = callinfo.name
        self.symdict = None
        self.sigdict = {}
        self.memdict = {}
        self.skipname = skipname
        self.name = self.__name__ = name
        # flatten, but keep BlockInstance objects
        self.subs = _flatten(func(*args, **kwargs))
        self._verifySubs()
        self._updateNamespaces()
        self.verilog_code = self.vhdl_code = None
        self.sim = None
        if hasattr(deco, 'verilog_code'):
            self.verilog_code = _UserVerilogCode(deco.verilog_code, self.symdict, func.__name__,
                                                 func, srcfile, srcline)
        if hasattr(deco, 'vhdl_code'):
            self.vhdl_code = _UserVhdlCode(deco.vhdl_code, self.symdict, func.__name__,
                                           func, srcfile, srcline)
        if hasattr(deco, 'verilog_instance'):
            self.verilog_code = _UserVerilogInstance(deco.vhdl_instance, self.symdict, func.__name__,
                                                     func, srcfile, srcline)
        if hasattr(deco, 'vhdl_instance'):
            self.vhdl_code = _UserVhdlInstance(deco.vhdl_instance, self.symdict, func.__name__,
                                               func, srcfile, srcline)
        self._config_sim = {'trace': False}

    def _verifySubs(self):
        for inst in self.subs:
            if not isinstance(inst, (_Block, _Instantiator, Cosimulation)):
                raise BlockError(_error.ArgType % (self.name,))
            if isinstance(inst, (_Block, _Instantiator)):
                if not inst.modctxt:
                    raise BlockError(_error.InstanceError % (self.name, inst.callername))

    def _updateNamespaces(self):
        # dicts to keep track of objects used in Instantiator objects
        usedsigdict = {}
        usedlosdict = {}
        for inst in self.subs:
            # the symdict of a block instance is defined by
            # the call context of its instantiations
            if isinstance(inst, Cosimulation):
                continue  # ignore
            if self.symdict is None:
                self.symdict = inst.callinfo.symdict
            if isinstance(inst, _Instantiator):
                usedsigdict.update(inst.sigdict)
                usedlosdict.update(inst.losdict)
        if self.symdict is None:
            self.symdict = {}
        # Special case: due to attribute reference transformation, the
        # sigdict and losdict from Instantiator objects may contain new
        # references. Therefore, update the symdict with them.
        # To be revisited.
        self.symdict.update(usedsigdict)
        self.symdict.update(usedlosdict)
        # Infer sigdict and memdict, with compatibility patches from _extractHierarchy
        for n, v in self.symdict.items():
            if isinstance(v, _Signal):
                self.sigdict[n] = v
                if n in usedsigdict:
                    v._markUsed()
            if _isListOfSigs(v):
                m = _makeMemInfo(v)
                self.memdict[n] = m
                if n in usedlosdict:
                    m._used = True

    def _inferInterface(self):
        from myhdl.conversion._analyze import _analyzeTopFunc
        intf = _analyzeTopFunc(self.func, *self.args, **self.kwargs)
        self.argnames = intf.argnames
        self.argdict = intf.argdict

    # Public methods
    # The puropse now is to define the API, optimizations later

    def  _clear(self):
        """ Clear a number of 'global' attributes.
        This is a workaround function for cleaning up before converts.
        """
        # workaround: elaborate again for the side effect on signal attibutes
        # TODO: jb -> jck: unfortunately this may/will also take twice as long, which for big designs matters!
        # and second it will print every user debug message twice cluttering the console output
        # so there must be a better way than this *lazy* workaround
        # maybe later ...
        if isboundmethod(self.func):
            if hasattr(self, 'isHdlClass'):
                # if present it will be `True`, even if it is `False` :)
                # An HdlClass object's hdl() method does not take any args nor kwargs
                # all ports/signals (must) have resolved in the `__iniy__()` call
                self.func()
            else:
                self.func(*self.args, **self.kwargs)
        else:
            self.func(*self.args, **self.kwargs)

        # reset number of calls in all blocks
        for b in myhdl._simulator._blocks:
            b.calls = 0

    def verify_convert(self):
        self._clear()
        return myhdl.conversion.verify(self)

    def analyze_convert(self):
        self._clear()
        return myhdl.conversion.analyze(self)

    def convert(self, hdl='Verilog', **kwargs):
        """Converts this BlockInstance to another HDL

        Args:
            hdl (Optional[str]): Target HDL. Defaults to Verilog

            path (Optional[str]): Destination folder. Defaults to current
                working dir.

            name (Optional[str]): Module and output file name. Defaults to
                `self.mod.__name__`

            initial_vales(Optional[bool(), str]): 
                Verilog: False: no initial values
                         True: all initial values, using initial blocks for memories
                         'skip_zero_mem_init': same as for `True` except no initial blocks are 
                                               generated for memories where all values are zero,
                                               which is the default at start-up of (most or all) FPGAs
                VHDL: True or False only ('skip_zero_mem_init' will be treated as `True` ...)

            trace(Optional[bool]): Verilog only. Whether the testbench should
                dump all signal waveforms. Defaults to False.

            testbench (Optional[bool]): Verilog only. Specifies whether a
                testbench should be created. Defaults to True.

            timescale(Optional[str]): Verilog only. Defaults to '1ns/10ps'
        """

        self._clear()

        if hdl.lower() == 'vhdl':
            converter = myhdl.conversion._toVHDL.toVHDL
        elif hdl.lower() == 'verilog':
            converter = myhdl.conversion._toVerilog.toVerilog
        else:
            raise BlockInstanceError('unknown hdl %s' % hdl)

        conv_attrs = {}
        if 'name' in kwargs:
            conv_attrs['name'] = kwargs.pop('name')
        conv_attrs['directory'] = kwargs.pop('path', '')
        if hdl.lower() == 'verilog':
            conv_attrs['no_testbench'] = not kwargs.pop('testbench', True)
            conv_attrs['timescale'] = kwargs.pop('timescale', '1ns/10ps')
            conv_attrs['trace'] = kwargs.pop('trace', False)

        conv_attrs.update(kwargs)
        for k, v in conv_attrs.items():
            setattr(converter, k, v)
        return converter(self)

    def config_sim(self, trace=False, **kwargs):
        self._config_sim['trace'] = trace
        if trace:
            for k, v in kwargs.items():
                setattr(myhdl.traceSignals, k, v)
            myhdl.traceSignals(self)

    def run_sim(self, duration=None, quiet=0):
        if self.sim is None:
            sim = self
            # if self._config_sim['trace']:
            #    sim = myhdl.traceSignals(self)
            self.sim = myhdl._Simulation.Simulation(sim)
        self.sim.run(duration, quiet)

    def quit_sim(self):
        if self.sim is not None:
            self.sim.quit()




import os
path = os.path
import unittest
from unittest import TestCase
import random
from random import randrange
random.seed(2)

from myhdl import (block, Signal, intbv, delay, instance, StopSimulation)
from myhdl._Simulation import Simulation

from .util import setupCosimulation


@block
def binaryOps(
              Bitand,
              Bitor,
              Bitxor,
              FloorDiv,
              LeftShift,
              Mod,
              Mul,
              Pow,
              RightShift,
              Sub,
              Sum,
              EQ,
              NE,
              LT,
              GT,
              LE,
              GE,
              And,
              Or,
              left, right):

    @instance
    def comb():
        while 1:
            yield left, right
            Bitand.next = left & right
            Bitor.next = left | right
            Bitxor.next = left ^ right
            if right != 0:
                FloorDiv.next = left // right
            if left < 256 and right < 40:
                LeftShift.next = left << right
            if right != 0:
                Mod.next = left % right
            Mul.next = left * right
            # Icarus doesn't support ** yet
            # if left < 256 and right < 40:
            #    Pow.next = left ** right
            Pow.next = 0
            RightShift.next = left >> right
            if left >= right:
                Sub.next = left - right
            Sum.next = left + right
            EQ.next = left == right
            NE.next = left != right
            LT.next = left < right
            GT.next = left > right
            LE.next = left <= right
            GE.next = left >= right
            And.next = bool(left) and bool(right)
            Or.next = bool(left) or bool(right)

    return comb


@block
def binaryOps_v(name,
                Bitand,
                Bitor,
                Bitxor,
                FloorDiv,
                LeftShift,
                Mod,
                Mul,
                Pow,
                RightShift,
                Sub,
                Sum,
                EQ,
                NE,
                LT,
                GT,
                LE,
                GE,
                And,
                Or,
                left, right):
    return setupCosimulation(**locals())


class TestBinaryOps(TestCase):

    def binaryBench(self, m, n):

        M = 2 ** m
        N = 2 ** n

        left = Signal(intbv(0)[m:])
        right = Signal(intbv(0)[n:])
        Bitand = Signal(intbv(0)[max(m, n):])
        Bitand_v = Signal(intbv(0)[max(m, n):])
        Bitor = Signal(intbv(0)[max(m, n):])
        Bitor_v = Signal(intbv(0)[max(m, n):])
        Bitxor = Signal(intbv(0)[max(m, n):])
        Bitxor_v = Signal(intbv(0)[max(m, n):])
        FloorDiv = Signal(intbv(0)[m:])
        FloorDiv_v = Signal(intbv(0)[m:])
        LeftShift = Signal(intbv(0)[64:])
        LeftShift_v = Signal(intbv(0)[64:])
        Mod = Signal(intbv(0)[m:])
        Mod_v = Signal(intbv(0)[m:])
        Mul = Signal(intbv(0)[m + n:])
        Mul_v = Signal(intbv(0)[m + n:])
        Pow = Signal(intbv(0)[64:])
        Pow_v = Signal(intbv(0)[64:])
        RightShift = Signal(intbv(0)[m:])
        RightShift_v = Signal(intbv(0)[m:])
        Sub = Signal(intbv(0)[max(m, n):])
        Sub_v = Signal(intbv(0)[max(m, n):])
        Sum = Signal(intbv(0)[max(m, n) + 1:])
        Sum_v = Signal(intbv(0)[max(m, n) + 1:])
        EQ, NE, LT, GT, LE, GE = [Signal(bool()) for __ in range(6)]
        EQ_v, NE_v, LT_v, GT_v, LE_v, GE_v = [Signal(bool()) for __ in range(6)]
        And, Or = [Signal(bool()) for __ in range(2)]
        And_v, Or_v, = [Signal(bool()) for __ in range(2)]

        binops = binaryOps(
                           Bitand,
                           Bitor,
                           Bitxor,
                           FloorDiv,
                           LeftShift,
                           Mod,
                           Mul,
                           Pow,
                           RightShift,
                           Sub,
                           Sum,
                           EQ,
                           NE,
                           LT,
                           GT,
                           LE,
                           GE,
                           And,
                           Or,
                           left,
                           right).convert(hdl='Verilog')
        binops_v = binaryOps_v(binaryOps.__name__,
                               Bitand_v,
                               Bitor_v,
                               Bitxor_v,
                               FloorDiv_v,
                               LeftShift_v,
                               Mod_v,
                               Mul_v,
                               Pow_v,
                               RightShift_v,
                               Sub_v,
                               Sum_v,
                               EQ_v,
                               NE_v,
                               LT_v,
                               GT_v,
                               LE_v,
                               GE_v,
                               And_v,
                               Or_v,
                               left, right)

        def stimulus():
            for i in range(min(M, N)):
                # print i
                left.next = intbv(i)
                right.next = intbv(i)
                yield delay(10)
            for i in range(100):
                left.next = randrange(M)
                right.next = randrange(N)
                yield delay(10)
            for j, k in ((0, 0), (0, N - 1), (M - 1, 0), (M - 1, N - 1)):
                left.next = j
                right.next = k
                yield delay(10)

        def check():
            while 1:
                yield left, right
                yield delay(1)
                # print "%s %s %s %s" % (left, right, Or, Or_v)
                self.assertEqual(Bitand, Bitand_v)
                self.assertEqual(Bitor, Bitor_v)
                self.assertEqual(Bitxor, Bitxor_v)
                self.assertEqual(FloorDiv, FloorDiv_v)
                self.assertEqual(LeftShift, LeftShift_v)
                self.assertEqual(Mod, Mod_v)
                self.assertEqual(Mul, Mul_v)
                # self.assertEqual(Pow, Pow_v)
                self.assertEqual(RightShift, RightShift_v)
                self.assertEqual(Sub, Sub_v)
                self.assertEqual(Sum, Sum_v)
                self.assertEqual(EQ, EQ_v)
                self.assertEqual(NE, NE_v)
                self.assertEqual(LT, LT_v)
                self.assertEqual(GT, GT_v)
                self.assertEqual(LE, LE_v)
                self.assertEqual(GE, GE_v)
                self.assertEqual(And, And_v)
                self.assertEqual(Or, Or_v)

        return binops, binops_v, stimulus(), check()

    def testBinaryOps(self):
        for m, n in ((4, 4,), (5, 3), (2, 6), (8, 7)):
            sim = self.binaryBench(m, n)
            Simulation(sim).run()


@block
def multiOps(
              Bitand,
              Bitor,
              Bitxor,
              And,
              Or,
              argm, argn, argp):

    @instance
    def comb():
        while 1:
            yield argm, argn, argp
            Bitand.next = argm & argn & argp
            Bitor.next = argm | argn | argp
            Bitxor.next = argm ^ argn ^ argp
            And.next = bool(argm) and bool(argn) and bool(argp)
            Or.next = bool(argm) and bool(argn) and bool(argp)

    return comb


@block
def multiOps_v(name,
                Bitand,
                Bitor,
                Bitxor,
                And,
                Or,
                argm, argn, argp):

    return setupCosimulation(**locals())


class TestMultiOps(TestCase):

    def multiBench(self, m, n, p):

        M = 2 ** m
        N = 2 ** n
        P = 2 ** p

        argm = Signal(intbv(0)[m:])
        argn = Signal(intbv(0)[n:])
        argp = Signal(intbv(0)[p:])
        Bitand = Signal(intbv(0)[max(m, n, p):])
        Bitand_v = Signal(intbv(0)[max(m, n, p):])
        Bitor = Signal(intbv(0)[max(m, n, p):])
        Bitor_v = Signal(intbv(0)[max(m, n, p):])
        Bitxor = Signal(intbv(0)[max(m, n, p):])
        Bitxor_v = Signal(intbv(0)[max(m, n, p):])
        And, Or = [Signal(bool()) for __ in range(2)]
        And_v, Or_v, = [Signal(bool()) for __ in range(2)]

        multiops = multiOps(
                           Bitand,
                           Bitor,
                           Bitxor,
                           And,
                           Or,
                           argm,
                           argn,
                           argp).convert(hdl='Verilog')
        multiops_v = multiOps_v(multiOps.__name__,
                                Bitand_v,
                                Bitor_v,
                                Bitxor_v,
                                And_v,
                                Or_v,
                                argm, argn, argp)

        def stimulus():
            for i in range(min(M, N, P)):
                # print i
                argm.next = intbv(i)
                argn.next = intbv(i)
                argp.next = intbv(i)
                yield delay(10)
            for i in range(100):
                argm.next = randrange(M)
                argn.next = randrange(N)
                argp.next = randrange(P)
                yield delay(10)
            for j, k, l in ((0, 0, 0), (0, 0, P - 1), (0, N - 1, P - 1),
                            (M - 1, 0, 0), (M - 1, 0, P - 1), (M - 1, N - 1, 0),
                            (0, N - 1, 0), (M - 1, N - 1, P - 1)):
                argm.next = j
                argn.next = k
                argp.next = l
                yield delay(10)

        def check():
            while 1:
                yield argm, argn, argp
                yield delay(1)
                # print "%s %s %s %s %s" % (argm, argn, argp, Bitxor, Bitxor_v)
                self.assertEqual(Bitand, Bitand_v)
                self.assertEqual(Bitor, Bitor_v)
                self.assertEqual(Bitxor, Bitxor_v)
                self.assertEqual(And, And_v)
                self.assertEqual(Or, Or_v)

        return multiops, multiops_v, stimulus(), check()

    def testMultiOps(self):
        for m, n, p in ((4, 4, 4,), (5, 3, 2), (3, 4, 6), (3, 7, 4)):
            sim = self.multiBench(m, n, p)
            Simulation(sim).run()


@block
def unaryOps(
             Not,
             Invert,
             UnaryAdd,
             UnarySub,
             arg):

    @instance
    def comb():
        while 1:
            yield arg
            Not.next = not arg
            Invert.next = ~arg
            UnaryAdd.next = +arg
            UnarySub.next = - -arg

    return comb


@block
def unaryOps_v(name, Not, Invert, UnaryAdd, UnarySub, arg):
    return setupCosimulation(**locals())


class TestUnaryOps(TestCase):

    def unaryBench(self, m):

        M = 2 ** m

        arg = Signal(intbv(0)[m:])
        Not = Signal(bool(0))
        Not_v = Signal(bool(0))
        Invert = Signal(intbv(0)[m:])
        Invert_v = Signal(intbv(0)[m:])
        UnaryAdd = Signal(intbv(0)[m:])
        UnaryAdd_v = Signal(intbv(0)[m:])
        UnarySub = Signal(intbv(0)[m:])
        UnarySub_v = Signal(intbv(0)[m:])

        unaryops = unaryOps(
                             Not,
                             Invert,
                             UnaryAdd,
                             UnarySub,
                             arg).convert(hdl='Verilog')
        unaryops_v = unaryOps_v(unaryOps.__name__,
                                Not_v,
                                Invert_v,
                                UnaryAdd_v,
                                UnarySub_v,
                                arg)

        def stimulus():
            for i in range(M):
                arg.next = intbv(i)
                yield delay(10)
            for i in range(100):
                arg.next = randrange(M)
                yield delay(10)
            raise StopSimulation

        def check():
            while 1:
                yield arg
                yield delay(1)
                self.assertEqual(Not, Not_v)
                self.assertEqual(Invert, Invert_v)
                self.assertEqual(UnaryAdd, UnaryAdd_v)
                self.assertEqual(UnarySub, UnarySub_v)

        return unaryops, unaryops_v, stimulus(), check()

    def testUnaryOps(self):
        for m in (4, 7):
            sim = self.unaryBench(m)
            Simulation(sim).run()


@block
def augmOps(
              Bitand,
              Bitor,
              Bitxor,
              FloorDiv,
              LeftShift,
              Mod,
              Mul,
              RightShift,
              Sub,
              Sum,
              left, right):

    @instance
    def comb():
        var = intbv(0)[max(64, len(left) + len(right)):]
        while 1:
            yield left, right
            var[:] = left
            var &= right
            Bitand.next = var
            var[:] = left
            var |= right
            Bitor.next = var
            var[:] = left
            var ^= left
            Bitxor.next = var
            if right != 0:
                var[:] = left
                var //= right
                FloorDiv.next = var
            if left < 256 and right < 40:
                var[:] = left
                var <<= right
                LeftShift.next = var
            if right != 0:
                var[:] = left
                var %= right
                Mod.next = var
            var[:] = left
            var *= right
            Mul.next = var
            var[:] = left
            var >>= right
            RightShift.next = var
            if left >= right:
                var[:] = left
                var -= right
                Sub.next = var
            var[:] = left
            var += right
            Sum.next = var

    return comb


@block
def augmOps_v(name,
                Bitand,
                Bitor,
                Bitxor,
                FloorDiv,
                LeftShift,
                Mod,
                Mul,
                RightShift,
                Sub,
                Sum,
                left, right):
    return setupCosimulation(**locals())


class TestAugmOps(TestCase):

    def augmBench(self, m, n):

        M = 2 ** m
        N = 2 ** n

        left = Signal(intbv(0)[m:])
        right = Signal(intbv(0)[n:])
        Bitand = Signal(intbv(0)[max(m, n):])
        Bitand_v = Signal(intbv(0)[max(m, n):])
        Bitor = Signal(intbv(0)[max(m, n):])
        Bitor_v = Signal(intbv(0)[max(m, n):])
        Bitxor = Signal(intbv(0)[max(m, n):])
        Bitxor_v = Signal(intbv(0)[max(m, n):])
        FloorDiv = Signal(intbv(0)[m:])
        FloorDiv_v = Signal(intbv(0)[m:])
        LeftShift = Signal(intbv(0)[64:])
        LeftShift_v = Signal(intbv(0)[64:])
        Mod = Signal(intbv(0)[m:])
        Mod_v = Signal(intbv(0)[m:])
        Mul = Signal(intbv(0)[m + n:])
        Mul_v = Signal(intbv(0)[m + n:])
        RightShift = Signal(intbv(0)[m:])
        RightShift_v = Signal(intbv(0)[m:])
        Sub = Signal(intbv(0)[max(m, n):])
        Sub_v = Signal(intbv(0)[max(m, n):])
        Sum = Signal(intbv(0)[max(m, n) + 1:])
        Sum_v = Signal(intbv(0)[max(m, n) + 1:])

        augmops = augmOps(
                           Bitand,
                           Bitor,
                           Bitxor,
                           FloorDiv,
                           LeftShift,
                           Mod,
                           Mul,
                           RightShift,
                           Sub,
                           Sum,
                           left, right).convert(hdl='Verilog')
        augmops_v = augmOps_v(augmOps.__name__,
                               Bitand_v,
                               Bitor_v,
                               Bitxor_v,
                               FloorDiv_v,
                               LeftShift_v,
                               Mod_v,
                               Mul_v,
                               RightShift_v,
                               Sub_v,
                               Sum_v,
                               left, right)

        def stimulus():
            for i in range(min(M, N)):
                # print i
                left.next = intbv(i)
                right.next = intbv(i)
                yield delay(10)
            for i in range(100):
                left.next = randrange(M)
                right.next = randrange(N)
                yield delay(10)
            for j, k in ((0, 0), (0, N - 1), (M - 1, 0), (M - 1, N - 1)):
                left.next = j
                right.next = k
                yield delay(10)

        def check():
            while 1:
                yield left, right
                yield delay(1)
                # print "%s %s %s %s" % (left, right, Or, Or_v)
                self.assertEqual(Bitand, Bitand_v)
                self.assertEqual(Bitor, Bitor_v)
                self.assertEqual(Bitxor, Bitxor_v)
                self.assertEqual(FloorDiv, FloorDiv_v)
                self.assertEqual(LeftShift, LeftShift_v)
                self.assertEqual(Mod, Mod_v)
                self.assertEqual(Mul, Mul_v)
                self.assertEqual(RightShift, RightShift_v)
                self.assertEqual(Sub, Sub_v)
                self.assertEqual(Sum, Sum_v)

        return augmops, augmops_v, stimulus(), check()

    def testAugmOps(self):
        for m, n in ((4, 4,), (5, 3), (2, 6), (8, 7)):
            sim = self.augmBench(m, n)
            Simulation(sim).run()


if __name__ == '__main__':
    unittest.main()




#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2008 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" module with the bin function.

"""
def _int2bitstring(num):
    if num == 0:
        return '0'
    if abs(num) == 1:
        return '1'
    bits = []
    p, q = divmod(num, 2)
    bits.append(str(q))
    while not (abs(p) == 1):
        p, q = divmod(p, 2)
        bits.append(str(q))
    bits.append('1')
    bits.reverse()
    return ''.join(bits)


def bin(num, width=0):
    """Return a binary string representation.

    num -- number to convert
    Optional parameter:
    width -- specifies the desired string (sign bit padding)
    """
    num = int(num)
    s = _int2bitstring(num)
    if width:
        pad = '0'
        if num < 0:
            pad = '1'
        return (width - len(s)) * pad + s
    return s



'''
Created on 19 okt. 2023

@author: josy
'''
from myhdl import (block, Signal, intbv, always_comb, always_seq , Constant,
                   instances)


@block
def scramble(Pattern, A, Y):
    NBR_BITS = len(Pattern)

    @always_comb
    def dsc():
        for i in range(NBR_BITS):
            if Pattern[i]:
                Y.next[i] = not A[i]
            else:
                Y.next[i] = A[i]

    return instances()


@block
def contrived(A, Y):
    WIDTH_D = len(A)
    PAT1 = Constant(intbv(0x42)[WIDTH_D:])
    PAT2 = Constant(intbv(0xbd)[WIDTH_D:])
    y1a2 = Signal(intbv(0)[WIDTH_D:])

    s1 = scramble(PAT1, A, y1a2)
    s2 = scramble(PAT2, y1a2, Y)

    return instances()


@block
def contrived2(A, Y):
    WIDTH_D = len(A)
    PAT = [Constant(intbv(0x42)[WIDTH_D:]), Constant(intbv(0xbd)[WIDTH_D:])]
    y1a2 = Signal(intbv(0)[WIDTH_D:])

    s1 = scramble(PAT[0], A, y1a2)
    s2 = scramble(PAT[1], y1a2, Y)

    return instances()


@block
def contrived3(WIDTH_D, Sel, Y):
    import random
    random.seed('We want repeatable randomness')

    A = [Constant(intbv(random.randint(1, 2 ** WIDTH_D - 1))[WIDTH_D:]) for __ in range(8)]

    @always_comb
    def cmux():
        Y.next = A[Sel]

    return instances()


@block
def contrived4(Clk, D, CE, Q):

    @always_seq(Clk.posedge, reset=None)
    def dff():
        if CE:
            Q.next = D

    return instances()


@block
def wrappercontrived4(Clk, D, Q):
    return contrived4(Clk, D, Constant(bool(1)), Q)


def test_contrived():
    WIDTH_D = 8
    A, Y = [Signal(intbv(0)[WIDTH_D:]) for __ in range(2)]
    assert contrived(A, Y).analyze_convert() == 0


def test_contrived2():
    WIDTH_D = 8
    A, Y = [Signal(intbv(0)[WIDTH_D:]) for __ in range(2)]
    assert contrived2(A, Y).analyze_convert() == 0


def test_contrived3():
    WIDTH_D = 8
    Y = Signal(intbv(0)[WIDTH_D:])
    Sel = Signal(intbv(0)[3:])
    assert contrived3(8, Sel, Y).analyze_convert() == 0


def test_contrived4():
    Clk, D , Q = [Signal(bool(0)) for __ in range(3)]
    CE = Signal(bool(0))
    assert contrived4(Clk, D, CE, Q).analyze_convert() == 0


def test_contrived4b():
    Clk, D , Q = [Signal(bool(0)) for __ in range(3)]
    assert wrappercontrived4(Clk, D, Q).analyze_convert() == 0


if __name__ == '__main__':

    from myhdl import delay, instance, StopSimulation

    @block
    def tb_contrived():
        WIDTH_D = 8
        A, Y = [Signal(intbv(0)[WIDTH_D:]) for __ in range(2)]

        # dut = contrived(A, Y)
        dut2 = contrived2(A, Y)

        @instance
        def stimulus():
            A.next = 0x42
            yield delay(10)
            A.next = Y
            yield delay(10)
            assert Y == 0x42

            raise StopSimulation

        return instances()

    def convert():
        WIDTH_D = 8
        A, Y = [Signal(intbv(0)[WIDTH_D:]) for __ in range(2)]
        Sel = Signal(intbv(0)[3:])
        Clk, D , Q = [Signal(bool(0)) for __ in range(3)]
        CE = Signal(bool(0))

        dfc = contrived(A, Y)
        dfc.convert(hdl='VHDL')
        dfc.convert(hdl='Verilog')

        dfc2 = contrived2(A, Y)
        dfc2.convert(hdl='VHDL')
        dfc2.convert(hdl='Verilog')

        dfc3 = contrived3(WIDTH_D, Sel, Y)
        dfc3.convert(hdl='VHDL')
        dfc3.convert(hdl='Verilog')

        dfc4 = contrived4(Clk, D, CE, Q)
        dfc4.convert(hdl='VHDL')
        dfc4.convert(hdl='Verilog')

        dfc5 = wrappercontrived4(Clk, D, Q)
        dfc5.convert(hdl='VHDL', name='contrived4b')
        dfc5.convert(hdl='Verilog', name='contrived4b')

    # dft = tb_contrived()
    # dft.config_sim(trace=True)
    # dft.run_sim()
    # print("Simulation passed")

    convert()




#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2008 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Run the concatunit tests. """
import operator
import random
from functools import reduce

import pytest

from myhdl import concat, intbv, Signal

random.seed(2)  # random, but deterministic


class TestConcat:

    bases = ("0", "1", "10101", "01010", "110", "011", "1001000100001011111000")
    extslist = [ ["0"], ["1"], ["00"], ["11"], ["000"], ["111"], ["1010101010"],
                 ["0", "1"], ["1", "0"], ["1", "01", "10"], ["11111", "001001"],
                 ["110001111101110", "10101110111001001", "111001101000101010"],
                 ["1100", "1", "01001", "0", "10", "01", "0", "0", "11", "1"  ]
               ]

    def ConcatToSizedBase(self, bases, extslist):
        for base, basestr in zip(bases, self.bases):
            for exts, extstr in zip(extslist, self.extslist):
                bv = concat(base, *exts)
                refstr = basestr + reduce(operator.add, extstr)
                reflen = len(refstr)
                ref = int(refstr, 2)
                assert bv == ref
                assert len(bv) == reflen

    def ConcatToUnsizedBase(self, bases, extslist):
        for base, basestr in zip(bases, self.bases):
            for exts, extstr in zip(extslist, self.extslist):
                bv = concat(base, *exts)
                refstr = basestr + reduce(operator.add, extstr)
                ref = int(refstr, 2)
                assert bv == ref
                assert len(bv) == 0

    def testConcatStringsToString(self):
        bases = self.bases
        extslist = self.extslist
        self.ConcatToSizedBase(bases, extslist)

    def testConcatStringsToInt(self):
        bases = [int(base, 2) for base in self.bases]
        extslist = self.extslist
        self.ConcatToUnsizedBase(bases, extslist)

    def testConcatStringsToSignalInt(self):
        bases = [Signal(int(base, 2)) for base in self.bases]
        extslist = self.extslist
        self.ConcatToUnsizedBase(bases, extslist)

    def testConcatStringsToIntbv(self):
        bases = [intbv(base) for base in self.bases]
        extslist = self.extslist
        self.ConcatToSizedBase(bases, extslist)

    def testConcatStringsToSignalIntbv(self):
        bases = [Signal(intbv(base)) for base in self.bases]
        extslist = self.extslist
        self.ConcatToSizedBase(bases, extslist)

    def testConcatStringsToBool(self):
        if type(bool) is not type:
            return
        bases = []
        for base in self.bases:
            if len(base) == 1:
                bases.append(bool(int(base)))
            else:
                bases.append(intbv(base))
        extslist = self.extslist
        self.ConcatToSizedBase(bases, extslist)

    def testConcatStringsToSignalBool(self):
        if type(bool) is not type:
            return
        bases = []
        for base in self.bases:
            if len(base) == 1:
                bases.append(Signal(bool(int(base))))
            else:
                bases.append(intbv(base))
        extslist = self.extslist
        self.ConcatToSizedBase(bases, extslist)

    def testConcatIntbvsToIntbv(self):
        bases = [intbv(base) for base in self.bases]
        extslist = []
        for exts in self.extslist:
            extslist.append([intbv(ext) for ext in exts])
        self.ConcatToSizedBase(bases, extslist)

    def testConcatSignalIntbvsToIntbv(self):
        bases = [intbv(base) for base in self.bases]
        extslist = []
        for exts in self.extslist:
            extslist.append([Signal(intbv(ext)) for ext in exts])
        self.ConcatToSizedBase(bases, extslist)

    def testConcatIntbvsToSignalIntbv(self):
        bases = [Signal(intbv(base)) for base in self.bases]
        extslist = []
        for exts in self.extslist:
            extslist.append([intbv(ext) for ext in exts])
        self.ConcatToSizedBase(bases, extslist)

    def testConcatSignalIntbvsToSignalIntbv(self):
        bases = [Signal(intbv(base)) for base in self.bases]
        extslist = []
        for exts in self.extslist:
            extslist.append([Signal(intbv(ext)) for ext in exts])
        self.ConcatToSizedBase(bases, extslist)

    def testConcatIntbvsToInt(self):
        bases = [int(base, 2) for base in self.bases]
        extslist = []
        for exts in self.extslist:
            extslist.append([intbv(ext) for ext in exts])
        self.ConcatToUnsizedBase(bases, extslist)

    def testConcatSignalIntbvsToInt(self):
        bases = [int(base, 2) for base in self.bases]
        extslist = []
        for exts in self.extslist:
            extslist.append([Signal(intbv(ext)) for ext in exts])
        self.ConcatToUnsizedBase(bases, extslist)

    def testConcatIntbvsToSignalInt(self):
        bases = [Signal(int(base, 2)) for base in self.bases]
        extslist = []
        for exts in self.extslist:
            extslist.append([intbv(ext) for ext in exts])
        self.ConcatToUnsizedBase(bases, extslist)

    def testConcatSignalIntbvsToSignalInt(self):
        bases = [Signal(int(base, 2)) for base in self.bases]
        extslist = []
        for exts in self.extslist:
            extslist.append([Signal(intbv(ext)) for ext in exts])
        self.ConcatToUnsizedBase(bases, extslist)

    def testConcatIntbvsBoolsToIntbv(self):
        if type(bool) is not type:
            return
        bases = [intbv(base) for base in self.bases]
        extslist = []
        for exts in self.extslist:
            newexts = []
            for ext in exts:
                if len(ext) == 1:
                    newexts.append(bool(int(ext)))
                else:
                    newexts.append(intbv(ext))
            extslist.append(newexts)
        self.ConcatToSizedBase(bases, extslist)

    def testConcatMixToSizedBase(self):
        bases = []
        for base in self.bases:
            seq = (base, intbv(base), Signal(intbv(base)))
            bases.append(random.choice(seq))
        extslist = []
        for exts in self.extslist:
            newexts = []
            for ext in exts:
                seq = (ext, intbv(ext), Signal(intbv(ext)))
                newexts.append(random.choice(seq))
            extslist.append(newexts)
        self.ConcatToSizedBase(bases, extslist)

    def testConcatMixToUnsizedBase(self):
        bases = []
        for base in self.bases:
            seq = (int(base, 2), Signal(int(base, 2)))
            bases.append(random.choice(seq))
        extslist = []
        for exts in self.extslist:
            newexts = []
            for ext in exts:
                seq = (ext, intbv(ext), Signal(intbv(ext)))
                newexts.append(random.choice(seq))
            extslist.append(newexts)
        self.ConcatToUnsizedBase(bases, extslist)

    def testConcatMixBoolToSizedBase(self):
        if type(bool) is not type:
            return
        bases = []
        for base in self.bases:
            seq = (base, intbv(base), Signal(intbv(base)))
            bases.append(random.choice(seq))
        extslist = []
        for exts in self.extslist:
            newexts = []
            for ext in exts:
                if len(ext) == 1:
                    seq = (ext, bool(int(ext)), Signal(bool(int(ext))))
                else:
                    seq = (ext, intbv(ext), Signal(intbv(ext)))
                newexts.append(random.choice(seq))
            extslist.append(newexts)
        self.ConcatToSizedBase(bases, extslist)

    def testWrongType(self):
        a = intbv(4)
        with pytest.raises(TypeError):
            concat(a, 5)

    def testUnsizedConcat(self):
        a = intbv(4)
        b = intbv(5)
        with pytest.raises(TypeError):
            concat(a, b)



"""
When an interface signal gets passed into a function, it
can get renamed to the name of the argument. When the
function is called multiple times, this causes name collisions """

from myhdl import (block, Signal, always_comb)


class AB:

    def __init__(self):
        self.a = Signal(bool(False))
        self.b = Signal(bool(False))


@block
def invert(sigin, sigout):

    @always_comb
    def foo():
        sigout.next = not sigin

    return foo


@block
def issue_134(ab_in, ab_out):
    """ Instantiate an inverter for each signal """
    inverta = invert(ab_in.a, ab_out.a)
    invertb = invert(ab_in.b, ab_out.b)
    return inverta, invertb


# @pytest.mark.xfail
def test_issue_134():
    """ check for port name collision"""
    assert issue_134(AB(), AB()).analyze_convert() == 0



import myhdl
from myhdl import *

from lfsr24 import lfsr24

def test_lfsr24():

    lfsr = Signal(modbv(0)[24:])
    enable = Signal(bool())
    clock = Signal(bool())
    reset = Signal(bool())
    
    dut = lfsr24(lfsr, enable, clock, reset)

    @instance
    def stimulus():
        enable.next = 0
        clock.next = 0
        reset.next = 0
        yield delay(10)
        reset.next = 1
        yield delay(10)
        reset.next = 0
        enable.next = 1
        for i in range(2**24 - 2):
            yield delay(10)
            clock.next = 1
            yield delay(10)
            clock.next = 0
            assert lfsr != 1
        yield delay(10)
        clock.next = 1
        yield delay(10)
        assert lfsr == 1 
        
    return dut, stimulus

if __name__ == '__main__':
    tb = Simulation(test_lfsr24())
    tb.run()

       



#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2008 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Run unit tests for Cosimulation """
import gc
import os
import random
import sys

if sys.platform == "win32":
    import msvcrt

from myhdl import Signal
from myhdl._Cosimulation import Cosimulation, CosimulationError, _error

if __name__ != '__main__':
    from helpers import raises_kind

random.seed(1)  # random, but deterministic

MAXLINE = 4096

exe = "python {0} ".format(os.path.abspath(__file__))

fromSignames = ['a', 'bb', 'ccc']
fromSizes = [1, 11, 63]
fromVals = [0x2, 0x43, 0x24]
fromSigs = {}
for s, v in zip(fromSignames, fromVals):
    fromSigs[s] = Signal(v)
toSignames = ['d', 'ee', 'fff', 'g']
toSizes = [32, 12, 3, 6]
toSigs = {}
for s in toSignames:
    toSigs[s] = Signal(0)
toVals = [0x3, 0x45, 0x14, 0x12]
toXVals = ["X00", "FZ3", "34XZ", "56U"]
allSigs = fromSigs.copy()
allSigs.update(toSigs)


def wtrf():
    if sys.platform != "win32":
        wt = int(os.environ['MYHDL_TO_PIPE'])
        rf = int(os.environ['MYHDL_FROM_PIPE'])
    else:
        wt = msvcrt.open_osfhandle(int(os.environ['MYHDL_TO_PIPE']), os.O_APPEND | os.O_TEXT)
        rf = msvcrt.open_osfhandle(int(os.environ['MYHDL_FROM_PIPE']), os.O_RDONLY | os.O_TEXT)

    return wt, rf


class TestCosimulation:

    def setup_method(self, method):
        gc.collect()

    def testWrongExe(self):
        with raises_kind(CosimulationError, _error.OSError):
            Cosimulation('bla -x 45')

    def testFromSignals(self):
        cosim = Cosimulation(exe + "cosimFromSignals", **allSigs)
        assert cosim._fromSignames == fromSignames
        assert cosim._fromSizes == fromSizes

    @staticmethod
    def cosimFromSignals():
        wt, rf = wtrf()
        buf = "FROM 00 "
        for s, w in zip(fromSignames, fromSizes):
            buf += "%s %s " % (s, w)
        os.write(wt, buf.encode())
        os.read(rf, MAXLINE)
        os.write(wt, b"TO 0000 a 1")
        os.read(rf, MAXLINE)
        os.write(wt, b"START")
        os.read(rf, MAXLINE)

    def testToSignals(self):
        cosim = Cosimulation(exe + "cosimToSignals", **toSigs)
        assert cosim._fromSignames == []
        assert cosim._fromSizes == []
        assert cosim._toSignames == toSignames
        assert cosim._toSizes == toSizes

    @staticmethod
    def cosimToSignals():
        wt, rf = wtrf()
        buf = "TO 00 "
        for s, w in zip(toSignames, toSizes):
            buf += "%s %s " % (s, w)
        os.write(wt, buf.encode())
        os.read(rf, MAXLINE)
        os.write(wt, b"FROM 0000")
        os.read(rf, MAXLINE)
        os.write(wt, b"START")
        os.read(rf, MAXLINE)

    def testFromToSignals(self):
        cosim = Cosimulation(exe + "cosimFromToSignals", **allSigs)
        assert cosim._fromSignames == fromSignames
        assert cosim._fromSizes == fromSizes
        assert cosim._toSignames == toSignames
        assert cosim._toSizes == toSizes

    @staticmethod
    def cosimFromToSignals():
        wt, rf = wtrf()
        buf = "FROM 00 "
        for s, w in zip(fromSignames, fromSizes):
            buf += "%s %s " % (s, w)
        os.write(wt, buf.encode())
        os.read(rf, MAXLINE)
        buf = "TO 00 "
        for s, w in zip(toSignames, toSizes):
            buf += "%s %s " % (s, w)
        os.write(wt, buf.encode())
        os.read(rf, MAXLINE)
        os.write(wt, b"START")
        os.read(rf, MAXLINE)

    def testTimeZero(self):
        with raises_kind(CosimulationError, _error.TimeZero):
            Cosimulation(exe + "cosimTimeZero", **allSigs)

    @staticmethod
    def cosimTimeZero():
        wt, rf = wtrf()
        buf = "TO 01 "
        for s, w in zip(fromSignames, fromSizes):
            buf += "%s %s " % (s, w)
        os.write(wt, buf.encode())

    def testNoComm(self):
        with raises_kind(CosimulationError, _error.NoCommunication):
            Cosimulation(exe + "cosimNoComm", **allSigs)

    @staticmethod
    def cosimNoComm():
        wt, rf = wtrf()
        os.write(wt, b"FROM 0000")
        os.read(rf, MAXLINE)
        os.write(wt, b"TO 0000")
        os.read(rf, MAXLINE)
        os.write(wt, b"START ")
        os.read(rf, MAXLINE)

    def testFromSignalsDupl(self):
        with raises_kind(CosimulationError, _error.DuplicateSigNames):
            Cosimulation(exe + "cosimFromSignalsDupl", **allSigs)

    @staticmethod
    def cosimFromSignalsDupl():
        wt, rf = wtrf()
        buf = "FROM 00 "
        for s, w in zip(fromSignames, fromSizes):
            buf += "%s %s " % (s, w)
        buf += "bb 5"
        os.write(wt, buf.encode())

    def testToSignalsDupl(self):
        with raises_kind(CosimulationError, _error.DuplicateSigNames):
            Cosimulation(exe + "cosimToSignalsDupl", **allSigs)

    @staticmethod
    def cosimToSignalsDupl():
        wt, rf = wtrf()
        buf = "TO 00 "
        for s, w in zip(toSignames, toSizes):
            buf += "%s %s " % (s, w)
        buf += "fff 6"
        os.write(wt, buf.encode())

    def testFromSignalVals(self):
        cosim = Cosimulation(exe + "cosimFromSignalVals", **allSigs)
        os.read(cosim._rt, MAXLINE)
        cosim._hasChange = 1
        cosim._put(0)

    @staticmethod
    def cosimFromSignalVals():
        wt, rf = wtrf()
        buf = "FROM 00 "
        for s, w in zip(fromSignames, fromSizes):
            buf += "%s %s " % (s, w)
        os.write(wt, buf.encode())
        os.read(rf, MAXLINE)
        os.write(wt, b"TO 0000 a 1")
        os.read(rf, MAXLINE)
        os.write(wt, b"START")
        os.read(rf, MAXLINE)
        os.write(wt, b"DUMMY")
        s = os.read(rf, MAXLINE)
        vals = [int(e, 16) for e in s.split()[1:]]
        assert vals == fromVals

    def testToSignalVals(self):
        cosim = Cosimulation(exe + "cosimToSignalVals", **allSigs)
        for n in toSignames:
            assert toSigs[n].next == 0
        cosim._get()
        for n, v in zip(toSignames, toVals):
            assert toSigs[n].next == v
        os.write(cosim._wf, b"DUMMY")
        cosim._getMode = 1
        cosim._get()
        for n in toSignames:
            assert toSigs[n].next == 0

    @staticmethod
    def cosimToSignalVals():
        wt, rf = wtrf()
        buf = "FROM 00 "
        for s, w in zip(fromSignames, fromSizes):
            buf += "%s %s " % (s, w)
        os.write(wt, buf.encode())
        os.read(rf, MAXLINE)
        buf = "TO 00 "
        for s, w in zip(toSignames, toSizes):
            buf += "%s %s " % (s, w)
        os.write(wt, buf.encode())
        os.read(rf, MAXLINE)
        os.write(wt, b"START")
        os.read(rf, MAXLINE)
        buf = "0 "
        for s, v in zip(toSignames, toVals):
            buf += s
            buf += " "
            buf += hex(v)[2:]
            buf += " "
        os.write(wt, buf.encode())
        os.read(rf, MAXLINE)
        buf = "0 "
        for s, v in zip(toSignames, toXVals):
            buf += s
            buf += " "
            buf += v
            buf += " "
        os.write(wt, buf.encode())


if __name__ == "__main__":
    getattr(TestCosimulation, sys.argv[1])()



from myhdl import (block, Signal, delay, always, always_comb,
                   instance, StopSimulation, conversion)


class HDLClass(object):

    @block
    def model(self, clock, input_interface, output_interface):

        internal_in = Signal(False)
        internal_out = Signal(False)

        @always_comb
        def assignments():
            internal_in.next = input_interface
            output_interface.next = internal_out

        @always(clock.posedge)
        def do_something():
            print('something')
            internal_out.next = internal_in.next

        return do_something, assignments


class InterfaceWithInstanceSignal(object):

    def __init__(self):

        self.internal_ins = [
            Signal(False), Signal(False), Signal(False), Signal(False)]
        self.internal_outs = [
            Signal(False), Signal(False), Signal(False), Signal(False)]

    @block
    def model(self, clock, input_interface, output_interface, index):

        internal_in = self.internal_ins[index]
        internal_out = self.internal_outs[index]

        @always_comb
        def assignments():
            internal_in.next = input_interface
            output_interface.next = internal_out

        @always(clock.posedge)
        def do_something():
            print('something')
            internal_out.next = internal_in.next

        return do_something, assignments


@block
def different_class_pipeline(clock, input_interface, output_interface):

    class_inst1 = HDLClass()
    class_inst2 = HDLClass()

    intermediate_interface = Signal(False)

    class_hdl_inst1 = class_inst1.model(
        clock, input_interface, intermediate_interface)

    class_hdl_inst2 = class_inst2.model(
        clock, intermediate_interface, output_interface)

    return class_hdl_inst1, class_hdl_inst2


@block
def common_class_pipeline(clock, input_interface, output_interface):

    class_inst = HDLClass()

    intermediate_interface = Signal(False)
    intermediate_interface_2 = Signal(False)
    intermediate_interface_3 = Signal(False)

    class_hdl_inst1 = class_inst.model(
        clock, input_interface, intermediate_interface)

    class_hdl_inst2 = class_inst.model(
        clock, intermediate_interface, intermediate_interface_2)

    class_hdl_inst3 = class_inst.model(
        clock, intermediate_interface_2, intermediate_interface_3)

    class_hdl_inst4 = class_inst.model(
        clock, intermediate_interface_3, output_interface)

    return class_hdl_inst1, class_hdl_inst2, class_hdl_inst3, class_hdl_inst4


@block
def interface_with_method_pipeline(clock, input_interface, output_interface):

    class_inst = InterfaceWithInstanceSignal()

    intermediate_interface = Signal(False)
    intermediate_interface_2 = Signal(False)
    intermediate_interface_3 = Signal(False)

    class_hdl_inst1 = class_inst.model(
        clock, input_interface, intermediate_interface, 0)

    class_hdl_inst2 = class_inst.model(
        clock, intermediate_interface, intermediate_interface_2, 1)

    class_hdl_inst3 = class_inst.model(
        clock, intermediate_interface_2, intermediate_interface_3, 2)

    class_hdl_inst4 = class_inst.model(
        clock, intermediate_interface_3, output_interface, 3)

    return class_hdl_inst1, class_hdl_inst2, class_hdl_inst3, class_hdl_inst4


@block
def bench(class_name='different_class'):

    clk = Signal(False)
    input_interface = Signal(False)
    output_interface = Signal(False)

    N = 20

    @instance
    def clkgen():

        clk.next = 0
        for dummy in range(N):
            yield delay(10)
            clk.next = not clk

        raise StopSimulation()

    if class_name == 'common_class':
        pipeline_inst = common_class_pipeline(
            clk, input_interface, output_interface)

    elif class_name == 'interface':
        pipeline_inst = interface_with_method_pipeline(
            clk, input_interface, output_interface)

    elif class_name == 'different_class':
        pipeline_inst = different_class_pipeline(
            clk, input_interface, output_interface)

    return pipeline_inst, clkgen


def test_multiple_class_single_method():
    assert conversion.verify(bench()) == 0


def test_single_class_single_method():
    assert conversion.verify(bench(class_name='common_class')) == 0


def test_single_interface_with_single_method():
    assert conversion.verify(bench(class_name='interface')) == 0




from myhdl import (block, Signal, modbv, delay, always_comb,
                   instance, StopSimulation, instances
                   )

import pytest


@block
def notVector0(
    a,
    z,
):

    @always_comb
    def outputs():
        z.next = a & ~0x1

    return instances()


@block
def notVector1(
    a,
    b,
    z,
):

    @always_comb
    def outputs():
        z.next = a & ~b

    return instances()


@block
def notVector2(
    a,
    z,
):

    @always_comb
    def outputs():
        z.next = a & +0xfe

    return instances()


@block
def notVector3(
    a,
    b,
    z,
):

    @always_comb
    def outputs():
        z.next = a & +b

    return instances()


@block
def notVector4(
    a,
    z,
):

    @always_comb
    def outputs():
        z.next = a & -0x2

    return instances()


@block
def notVectorBench0(rev=0):

    clk = Signal(bool(0))
    a = Signal(modbv(0)[8:])
    b = Signal(modbv(0)[8:])
    z = Signal(modbv(0)[8:])

    @instance
    def clkgen():
        clk.next = 1
        for dummy in range(400):
            yield delay(10)
            clk.next = not clk

    @instance
    def stimulus():
        a.next = 0x0
        if 3 == rev:
            b.next = 0xfe
        elif 5 == rev:
            b.next = 0x2
        else:
            b.next = 0x1
        yield clk.posedge
        yield clk.posedge
        a.next = 0x0
        yield clk.posedge
        a.next = 0x1
        yield clk.posedge
        a.next = 0x2
        yield clk.posedge
        a.next = 0x3
        yield clk.posedge
        a.next = 0xff
        yield clk.posedge

        raise StopSimulation

    @instance
    def check():
        yield clk.posedge
        yield clk.posedge
        yield clk.posedge
        assert z == 0x0
        yield clk.posedge
        assert z == 0x0
        yield clk.posedge
        assert z == 0x2
        yield clk.posedge
        assert z == 0x2
        yield clk.posedge
        assert z == 0xfe
        yield clk.posedge

    if 1 == rev:
        i_notVector = notVector1(a, b, z)
    elif 2 == rev:
        i_notVector = notVector2(a, z)
    elif 3 == rev:
        i_notVector = notVector3(a, b, z)
    elif 4 == rev:
        i_notVector = notVector4(a, z)
    else:
        i_notVector = notVector0(a, z)

    return i_notVector


def test_notVector0():
    sim = notVectorBench0(0)
    sim.run_sim()


def test_notVector0Inst():

    a = Signal(modbv(0)[8:])
    z = Signal(modbv(0)[8:])

    i_dut = notVector0(a, z)
    assert i_dut.analyze_convert() == 0


@pytest.mark.filterwarnings("ignore:Signal is driven")
def test_notVector0_convert():
    i_dut = notVectorBench0(0)
    assert i_dut.analyze_convert() == 0
    # assert i_dut.verify_convert() == 0


def test_notVector1():
    sim = notVectorBench0(1)
    sim.run_sim()


def test_notVector1Inst():

    a = Signal(modbv(0)[8:])
    b = Signal(modbv(0)[8:])
    z = Signal(modbv(0)[8:])

    i_dut = notVector1(a, b, z)
    assert i_dut.analyze_convert() == 0


def test_notVector1_convert():
    i_dut = notVectorBench0(1)
    assert i_dut.analyze_convert() == 0
    # assert i_dut.verify_convert() == 0


def test_notVector2():
    sim = notVectorBench0(2)
    sim.run_sim()


def test_notVector2Inst():

    a = Signal(modbv(0)[8:])
    z = Signal(modbv(0)[8:])

    i_dut = notVector2(a, z)
    assert i_dut.analyze_convert() == 0


@pytest.mark.filterwarnings("ignore:Signal is driven")
def test_notVector2_convert():
    i_dut = notVectorBench0(2)
    assert i_dut.analyze_convert() == 0
    # assert i_dut.verify_convert() == 0


def test_notVector3():
    sim = notVectorBench0(3)
    sim.run_sim()


def test_notVector3Inst():

    a = Signal(modbv(0)[8:])
    b = Signal(modbv(0)[8:])
    z = Signal(modbv(0)[8:])

    i_dut = notVector3(a, b, z)
    assert i_dut.analyze_convert() == 0


def test_notVector3_convert():
    i_dut = notVectorBench0(3)
    assert i_dut.analyze_convert() == 0
    # assert i_dut.verify_convert() == 0


def test_notVector4():
    sim = notVectorBench0(4)
    sim.run_sim()


def test_notVector4Inst():

    a = Signal(modbv(0)[8:])
    z = Signal(modbv(0)[8:])

    i_dut = notVector4(a, z)
    assert i_dut.analyze_convert() == 0


@pytest.mark.filterwarnings("ignore:Signal is driven")
def test_notVector4_convert():
    i_dut = notVectorBench0(4)
    assert i_dut.analyze_convert() == 0
    # assert i_dut.verify_convert() == 0




from myhdl import (block, Signal, intbv, instance, delay,
                   ConcatSignal, always_comb)


@block
def adapter(o_err, i_err, o_spec, i_spec):

    nomatch = Signal(bool(0))
    other = Signal(bool(0))

    o_err_bits = []
    for s in o_spec:
        if s == 'other':
            o_err_bits.append(other)
        elif s == 'nomatch':
            o_err_bits.append(nomatch)
        else:
            bit = i_err(i_spec[s])
            o_err_bits.append(bit)
    o_err_vec = ConcatSignal(*o_err_bits)

    other_bits = []
    for s, i in i_spec.items():
        if s in o_spec:
            continue
        bit = i_err(i)
        other_bits.append(bit)
    other_vec = ConcatSignal(*other_bits)

    @always_comb
    def assign():
        nomatch.next = 0
        other.next = (other_vec != 0)
        o_err.next = o_err_vec

    return assign


@block
def bench_adapter(hdl=None):
    o_spec = ('c', 'a', 'other', 'nomatch')
    i_spec = { 'a': 1, 'b': 2, 'c': 0, 'd': 3, 'e': 4, 'f': 5, }

    o_err = Signal(intbv(0)[4:])
    i_err = Signal(intbv(0)[6:])

    if hdl:
        dut = adapter(o_err, i_err, o_spec, i_spec).convert(hdl=hdl)
    else:
        dut = adapter(o_err, i_err, o_spec, i_spec)

    N = 2 ** len(i_err)

    @instance
    def stimulus():
        for i in range(N):
            i_err.next = i
            yield delay(10)
            assert o_err[0] == 0
            assert o_err[1] == (i_err[2] | i_err[3] | i_err[4] | i_err[5])
            assert o_err[2] == i_err[1]
            assert o_err[3] == i_err[0]
            print(o_err)

    return dut, stimulus


def test_adapter():
    assert bench_adapter().verify_convert() == 0


bench_adapter('Verilog')
bench_adapter('VHDL')



import os
path = os.path
from random import randrange

from myhdl import (block, Signal, intbv, delay, instance)
from myhdl.conversion import analyze
from myhdl import ConversionError
from myhdl.conversion._misc import _error


@block
def ForLoopError1(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in range(1, 4, -1):
                if a[i] == 1:
                    var += 1
            out.next = var

    return comb


@block
def LoopBench(LoopTest):

    a = Signal(intbv(-1)[16:])
    z = Signal(intbv(0)[16:])

    looptest_inst = LoopTest(a, z)
    data = tuple([randrange(2 ** min(i, 16)) for i in range(100)])

    @instance
    def stimulus():
        for i in range(100):
            a.next = data[i]
            yield delay(10)
            print(z)

    return stimulus, looptest_inst


def testForLoopError1():
    try:
        analyze(LoopBench(ForLoopError1))
    except ConversionError as e:
        assert e.kind == _error.Requirement
    else:
        assert False




#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2008 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Run the unit tests for bin """
import random
import sys
from random import randrange

from myhdl import bin

random.seed(1)  # random, but deterministic

SIZE = 100


def _int2bitstring(num):
    if num == 0:
        return '0'
    if abs(num) == 1:
        return '1'
    return _int2bitstring(num // 2) + _int2bitstring(num % 2)


def binref(num, width=0):
    """Return a binary string representation.

    num -- number to convert
    Optional parameter:
    width -- specifies the desired string (sign bit padding)
    """
    num = int(num)
    s = _int2bitstring(num)
    if width:
        pad = '0'
        if num < 0:
            pad = '1'
        return (width - len(s)) * pad + s
    return s


class TestBin:

    def testSmall(self):
        for i in range(-65, 65):
            assert bin(i) == binref(i)

    def testSmallWidth(self):
        for i in range(-65, 65):
            w = randrange(1, 8)
            assert bin(i, w) == binref(i, w)

    def testRandomInt(self):
        for __ in range(SIZE):
            i = randrange(-sys.maxsize, sys.maxsize)
            assert bin(i) == binref(i)

    def testRandomIntWidth(self):
        for __ in range(SIZE):
            w = randrange(1, 1000)
            i = randrange(-sys.maxsize, sys.maxsize)
            assert bin(i, w) == binref(i, w)

    def testRandomLong(self):
        for __ in range(SIZE):
            k = randrange(sys.maxsize)
            i = k + sys.maxsize
            assert bin(i) == binref(i)
            i = -k - sys.maxsize
            assert bin(i) == binref(i)

    def testRandomLongWith(self):
        for __ in range(SIZE):
            w = randrange(1, 1000)
            k = randrange(sys.maxsize)
            i = k + sys.maxsize
            assert bin(i, w) == binref(i, w)
            i = -k - sys.maxsize
            assert bin(i, w) == binref(i, w)



import os
path = os.path
import random
random.seed(2)

from myhdl import (block, intbv, instance, delay)

ACTIVE_LOW, INACTIVE_HIGH = bool(0), bool(1)


@block
def bug_1835792 ():
    """ Semicolon conversion

    """

    @instance
    def comb():
        v = intbv(0, min=-15, max=45)
        yield delay(10)
        print(v.min);
        print(v.max);

    return comb


def test_bug_1835792 ():
    assert bug_1835792().verify_convert() == 0




# -*- coding: utf-8 -*-
"""Failed VHDL code example
"""
from myhdl import (block, Signal, intbv, instance, delay, concat)


def unsigned(width, value=0, cls=intbv):
    """Create an unsigned signal based on a bitvector with the
    specified width and initial value.
    """
    return Signal(cls(value, 0, 2 ** width))


def signed(width, value=0, cls=intbv):
    """Create an signed signal based on a bitvector with the
    specified width and initial value.
    """
    return Signal(cls(value, -2 ** (width - 1), 2 ** (width - 1)))


flags = unsigned(4)
position = signed(28)


@block
def Logic(flags, position):

    conc = unsigned(32)

    @instance
    def doit():
        flags.next = 4
        position.next = 28
        yield delay(10)
        conc.next = concat(flags, position)
        yield delay(10)
        print(conc)

    return doit


def test_issue_10_2():
    assert Logic(flags, position).verify_convert() == 0



from myhdl import (block, Signal, intbv, ResetSignal, always_seq)


def shift_left(c, a, b):
    c.next = a << b


@block
def shifter(reset, clock, opa, opb, result):

    @always_seq(clock.posedge, reset=reset)
    def assign():
        shift_left(result, opa, opb)

    return assign


def convert():

    clock = Signal(bool(0))
    reset = ResetSignal(0, active=True, isasync=True)

    opa = Signal(intbv(0)[4:])
    opb = Signal(intbv(0)[4:])
    result = Signal(intbv(0)[10:])

    inst = shifter(reset, clock, opa, opb, result)
    inst.convert(hdl='VHDL')


if __name__ == '__main__':
    convert()



#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2008 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Run the unit tests for always_comb """
import random
from random import randrange

from myhdl import (AlwaysCombError, Signal, Simulation, StopSimulation, delay,
                   instances, intbv)
from myhdl._always_comb import _error, always_comb
from myhdl._Waiter import _SignalTupleWaiter, _SignalWaiter, _Waiter
from helpers import raises_kind
# random.seed(3) # random, but deterministic

QUIET = 1


def g():
    pass


x = Signal(0)


class TestAlwaysCombCompilation:

    def testArgIsFunction(self):
        h = 5
        with raises_kind(AlwaysCombError, _error.ArgType):
            always_comb(h)

    def testArgIsNormalFunction(self):

        def h():
            yield None

        with raises_kind(AlwaysCombError, _error.ArgType):
            always_comb(h)

    def testArgHasNoArgs(self):

        def h(n):
            return n

        with raises_kind(AlwaysCombError, _error.NrOfArgs):
            always_comb(h)

# #     def testScope(self):
# #         try:
# #             always_comb(g)
# #         except AlwaysCombError, e:
# #             self.assertEqual(e.kind, _error.Scope)
# #         else:
# #             self.fail()

    def testInfer1(self):
        a, c = [Signal(0) for __ in range(2)]
        u = 1

        def h():
            c.next = a
            v = u

        g = always_comb(h).gen
        i = g.gi_frame.f_locals['self']
        expected = set(['a'])
        assert i.inputs == expected

    def testInfer2(self):
        a, c = [Signal(0) for __ in range(2)]
        u = 1

        def h():
            c.next = x
            g = a

        g = always_comb(h).gen
        i = g.gi_frame.f_locals['self']
        expected = set(['a', 'x'])
        assert i.inputs == expected

    def testInfer3(self):
        a, c = [Signal(0) for __ in range(2)]
        u = 1

        def h():
            c.next = a + x + u
            a = 1

        g = always_comb(h).gen
        i = g.gi_frame.f_locals['self']
        expected = set(['x'])
        assert i.inputs == expected

    def testInfer4(self):
        a, c = [Signal(0) for __ in range(2)]
        u = 1

        def h():
            c.next = a + x + u
            x = 1

        g = always_comb(h).gen
        i = g.gi_frame.f_locals['self']
        expected = set(['a'])
        assert i.inputs == expected

    # def testInfer5(self):
    #     a, b, c, d = [Signal(0) for __ in range(4)]
    #
    #     def h():
    #         c.next += 1
    #         a += 1
    #     with raises_kind(AlwaysCombError, _error.SignalAsInout % set('c')):
    #         g = always_comb(h).gen
    #
    # def testInfer6(self):
    #     a, b, c, d = [Signal(0) for __ in range(4)]
    #
    #     def h():
    #         c.next = a
    #         x.next = c
    #     with raises_kind(AlwaysCombError, _error.SignalAsInout % set('c')):
    #         g = always_comb(h).gen

    def testInfer7(self):
        a, b, c = [Signal(0) for __ in range(3)]

        def h():
            c.next[a:0] = x[b:0]

        g = always_comb(h).gen
        i = g.gi_frame.f_locals['self']
        expected = set(['a', 'b', 'x'])
        assert i.inputs == expected

    def testInfer8(self):
        a, b, c = [Signal(0) for __ in range(3)]
        u = 1

        def h():
            v = 2
            c.next[8:1 + a + v] = x[4:b * 3 + u]

        g = always_comb(h).gen
        i = g.gi_frame.f_locals['self']
        expected = set(['a', 'b', 'x'])
        assert i.inputs == expected

    def testInfer9(self):
        a, b, c = [Signal(0) for __ in range(3)]

        def h():
            c.next[a - 1] = x[b - 1]

        g = always_comb(h).gen
        i = g.gi_frame.f_locals['self']
        expected = set(['a', 'b', 'x'])
        assert i.inputs == expected

    def testInfer10(self):
        a, b, c, d = [Signal(0) for __ in range(4)]

        def f(x, y, z):
            return 0

        def h():
            c.next = f(a, 2 * b, d * x)

        g = always_comb(h).gen
        i = g.gi_frame.f_locals['self']
        expected = set(['a', 'b', 'd', 'x'])
        assert i.inputs == expected

    def testEmbeddedFunction(self):
        a, b, c, d = [Signal(0) for __ in range(4)]
        u = 1

        def h():

            def g():
                e = b
                return e

            c.next = x
            g = a

        with raises_kind(AlwaysCombError, _error.EmbeddedFunction):
            g = always_comb(h)


class TestAlwaysCombSimulation1:

    def bench(self, function):

        clk = Signal(0)
        a = Signal(0)
        b = Signal(0)
        c = Signal(0)
        d = Signal(0)
        z = Signal(0)
        vectors = [intbv(j) for __ in range(32) for j in range(16)]
        random.shuffle(vectors)

        def combFunc():
            if __debug__:
                f = x
            x.next = function(a, b, c, d)

        comb = always_comb(combFunc)

        def clkGen():
            while 1:
                yield delay(10)
                clk.next ^= 1

        def logic():
            while 1:
                z.next = function(a, b, c, d)
                yield a, b, c, d

        def stimulus():
            for v in vectors:
                a.next = v[0]
                b.next = v[1]
                c.next = v[2]
                d.next = v[3]
                yield clk.posedge
                yield clk.negedge
                assert x == z
            raise StopSimulation("always_comb simulation test")

        return instances()

    def testAnd(self):

        def andFunction(a, b, c, d):
            return a & b & c & d

        Simulation(self.bench(andFunction)).run(quiet=QUIET)

    def testOr(self):

        def orFunction(a, b, c, d):
            return a | b | c | d

        Simulation(self.bench(orFunction)).run(quiet=QUIET)

    def testXor(self):

        def xorFunction(a, b, c, d):
            return a ^ b ^ c ^ d

        Simulation(self.bench(xorFunction)).run(quiet=QUIET)

    def testMux(self):

        def muxFunction(a, b, c, d):
            if c:
                return a
            else:
                return b

        Simulation(self.bench(muxFunction)).run(quiet=QUIET)

    def testLogic(self):

        def function(a, b, c, d):
            return not (a & (not b)) | ((not c) & d)

        Simulation(self.bench(function)).run(quiet=QUIET)


class TestAlwaysCombSimulation2:

    def bench(self, funcName="and"):

        clk = Signal(0)
        a = Signal(0)
        b = Signal(0)
        c = Signal(0)
        d = Signal(0)
        k = Signal(0)
        z = Signal(0)
        x = Signal(0)
        vectors = [intbv(j) for __ in range(32) for j in range(16)]
        random.shuffle(vectors)

        def andFunc():
            x.next = a & b & c & d

        def andGenFunc():
            while 1:
                z.next = a & b & c & d
                yield a, b, c, d

        def orFunc():
            x.next = a | b | c | d

        def orGenFunc():
            while 1:
                z.next = a | b | c | d
                yield a, b, c, d

        def logicFunc():
            x.next = not (a & (not b)) | ((not c) & d)

        def logicGenFunc():
            while 1:
                z.next = not (a & (not b)) | ((not c) & d)
                yield a, b, c, d

        def incFunc():
            x.next = k + 1

        def incGenFunc():
            while 1:
                z.next = k + 1
                yield k

        combFunc = eval(funcName + "Func")
        comb = always_comb(combFunc)
        genFunc = eval(funcName + "GenFunc")
        gen = genFunc()

        def clkGen():
            while 1:
                yield delay(10)
                clk.next ^= 1

        def stimulus():
            for v in vectors:
                a.next = v[0]
                b.next = v[1]
                c.next = v[2]
                d.next = v[3]
                k.next = v
                yield clk.posedge
                yield clk.negedge
                assert x == z
            raise StopSimulation("always_comb simulation test")

        return comb, gen, clkGen(), stimulus()

    def testAnd(self):
        Simulation(self.bench("and")).run(quiet=QUIET)

    def testOr(self):
        Simulation(self.bench("or")).run(quiet=QUIET)

    def testLogic(self):
        Simulation(self.bench("logic")).run(quiet=QUIET)

    def testInc(self):
        Simulation(self.bench("inc")).run(quiet=QUIET)


def SignalGen1(a, b, c, d, r):

    @always_comb
    def comb():
        r.next = a

    return comb


def SignalTupleGen1(a, b, c, d, r):

    @always_comb
    def comb():
        r.next = a + b + c

    return comb


class TestInferWaiter:

    def bench(self, MyHDLFunc, waiterType):

        a, b, c, d, r, s = [Signal(intbv(0)) for __ in range(6)]

        inst_r = MyHDLFunc(a, b, c, d, r)
        assert type(inst_r.waiter) == waiterType

        inst_s = MyHDLFunc(a, b, c, d, s)

        def stimulus():
            for dummy in range(1000):
                yield delay(randrange(1, 10))
                if randrange(2):
                    a.next = randrange(32)
                if randrange(2):
                    b.next = randrange(32)
                c.next = randrange(2)
                d.next = randrange(2)
            raise StopSimulation

        def check():
            while 1:
                yield a, b, c, r, s
                assert r == s

        return inst_r, _Waiter(inst_s.gen), _Waiter(stimulus()), _Waiter(check())

    def testSignal1(self):
        sim = Simulation(self.bench(SignalGen1, _SignalWaiter))
        sim.run()

    def testSignalTuple1(self):
        sim = Simulation(self.bench(SignalTupleGen1, _SignalTupleWaiter))
        sim.run()



import myhdl
from myhdl import *

def ram(dout, din, addr, we, clk, depth=128):
    """  Ram model """
    
    mem = [Signal(intbv(0)[8:]) for i in range(depth)]
    
    @always(clk.posedge)
    def write():
        if we:
            mem[addr].next = din
                
    @always_comb
    def read():
        dout.next = mem[addr]

    return write, read


dout = Signal(intbv(0)[8:])
dout_v = Signal(intbv(0)[8:])
din = Signal(intbv(0)[8:])
addr = Signal(intbv(0)[7:])
we = Signal(bool(0))
clk = Signal(bool(0))

def main():
    toVerilog.name = 'ram_1'
    toVerilog(ram, dout, din, addr, we, clk)
    toVHDL(ram, dout, din, addr, we, clk)
    
if __name__ == '__main__':
    main()



from myhdl import (block, Signal, intbv, modbv, always)

from myhdl import ConversionError
from myhdl.conversion._misc import _error
from myhdl.conversion import verify


@block
def sigAugmAssignUnsupported(z, a):

    @always(a)
    def comb():
        z.next += a

    return comb


def test_SigAugmAssignUnsupported():
    z = Signal(intbv(0)[8:])
    a = Signal(intbv(0)[8:])
    try:
        verify(sigAugmAssignUnsupported(z, a))
    except ConversionError as e:
        assert e.kind == _error.NotSupported
    else:
        assert False


@block
def modbvRange(z, a, b):

    @always(a, b)
    def comb():
        s = modbv(0, min=0, max=35)
        s[:] = a + b
        z.next = s

    return comb


def test_modbvRange():
    z = Signal(intbv(0)[8:])
    a = Signal(intbv(0)[4:])
    b = Signal(intbv(0)[4:])
    try:
        verify(modbvRange(z, a, b))
    except ConversionError as e:
        assert e.kind == _error.ModbvRange
    else:
        assert False


@block
def modbvSigRange(z, a, b):

    @always(a, b)
    def comb():
        z.next = a + b

    return comb


def test_modbvSigRange():
    z = Signal(modbv(0, min=0, max=42))
    a = Signal(intbv(0)[4:])
    b = Signal(intbv(0)[4:])
    try:
        verify(modbvSigRange(z, a, b))
    except ConversionError as e:
        assert e.kind == _error.ModbvRange
    else:
        assert False



from myhdl import (Signal, enum)


def issue_9():

    t_State = enum('foo', 'bar')

    assert (Signal(t_State.foo) == Signal(t_State.bar)) == False
    assert (Signal(t_State.foo) != Signal(t_State.bar)) == True
    assert (Signal(t_State.foo) == Signal(t_State.foo)) == True
    assert (Signal(t_State.foo) != Signal(t_State.foo)) == False


def test_issue_9():
    issue_9()



#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2009 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Module with utilility objects for MyHDL.

"""
import __future__
import ast
import sys
import inspect

from tokenize import generate_tokens, untokenize, INDENT
from io import StringIO


def _printExcInfo():
    kind, value = sys.exc_info()[:2]
    msg = str(kind)
    # msg = msg[msg.rindex('.')+1:]
    if str(value):
        msg += ": %s" % value
        print(msg, file=sys.stderr)

_isGenFunc = inspect.isgeneratorfunction


def _flatten(*args):
    arglist = []
    for arg in args:
        if isinstance(arg, (list, tuple, set)):
            for item in arg:
                arglist.extend(_flatten(item))
        else:
            arglist.append(arg)
    return arglist


def _isTupleOfInts(obj):
    if not isinstance(obj, tuple):
        return False
    for e in obj:
        if not isinstance(e, int):
            return False
    return True


def _dedent(s):
    """Dedent python code string."""

    result = [t[:2] for t in generate_tokens(StringIO(s).readline)]
    # set initial indent to 0 if any
    if result[0][0] == INDENT:
        result[0] = (INDENT, '')
    return untokenize(result)


def _makeAST(f):
    # Need to look at the flags used to compile the original function f and
    # pass these same flags to the compile() function. This ensures that
    # syntax-changing __future__ imports like print_function work correctly.
    orig_f_co_flags = f.__code__.co_flags
    # co_flags can contain various internal flags that we can't pass to
    # compile(), so strip them out here
    valid_flags = 0
    for future_feature in __future__.all_feature_names:
        feature = getattr(__future__, future_feature)
        valid_flags |= feature.compiler_flag
    s = inspect.getsource(f)
    s = _dedent(s)
    # use compile instead of ast.parse so that additional flags can be passed
    flags = ast.PyCF_ONLY_AST | (orig_f_co_flags & valid_flags)
    tree = compile(s, filename='<unknown>', mode='exec',
        flags=flags, dont_inherit=True)
    # tree = ast.parse(s)
    tree.sourcefile = inspect.getsourcefile(f)
    tree.lineoffset = inspect.getsourcelines(f)[1] - 1
    return tree


def _genfunc(gen):
    from myhdl._always_comb import _AlwaysComb
    from myhdl._always_seq import _AlwaysSeq
    from myhdl._always import _Always
    if isinstance(gen, (_AlwaysComb, _AlwaysSeq, _Always)):
        func = gen.func
    else:
        func = gen.genfunc
    return func



#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2016 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" myhdl _getHierarchy module.

"""

from myhdl._extractHierarchy import _Instance
from myhdl._block import _Block


class _Hierarchy(object):

    def __init__(self, name, modinst):
        self.top = modinst
        self.hierarchy = hierarchy = []
        self.absnames = absnames = {}
        _getHierarchyHelper(1, modinst, hierarchy)
        # compatibility with _extractHierarchy
        # walk the hierarchy to define relative and absolute names
        names = {}
        top_inst = hierarchy[0]
        obj, subs = top_inst.obj, top_inst.subs
        names[id(obj)] = name
        absnames[id(obj)] = None # this avoids starting every process label with the name of the entity
        for inst in hierarchy:
            obj, subs = inst.obj, inst.subs
            inst.name = names[id(obj)]
            tn = absnames[id(obj)]
            for sn, so in subs:
                names[id(so)] = sn
                # if sn is None:
                #     absnames[id(so)] = "%s" % (tn,)
                # else:
                #     absnames[id(so)] = "%s_%s" % (tn, sn)
                if sn is None:
                    absnames[id(so)] = tn
                else:
                    if tn is not None:
                        absnames[id(so)] = '_'.join((tn, sn))
                    else:
                        absnames[id(so)] = sn


def _getHierarchy(name, modinst):
    h = _Hierarchy(name, modinst)
    return h


def _getHierarchyHelper(level, modinst, hierarchy):
    subs = [(s.name, s) for s in modinst.subs]
    inst = _Instance(level, modinst, subs, modinst.sigdict, modinst.memdict)
    hierarchy.append(inst)
    for inst in modinst.subs:
        if isinstance(inst, _Block):
            _getHierarchyHelper(level + 1, inst, hierarchy)



import unittest
import os
path = os.path
from random import randrange

from myhdl import (block, Signal, intbv, delay, instances,
                   instance)
from myhdl._Simulation import Simulation

from .util import setupCosimulation

### test of constant wire support ###


# example from Frank Palazollo
@block
def or_gate(a, b, c):

    @instance
    def comb():
        while 1:
            c.next = a | b
            yield a, b

    return comb


@block
def my_bundle(p, q):
        r = Signal(bool(0))
        gen_or = or_gate(p, r, q)
        return instances()


# additional level of hierarchy
@block
def ConstWire2(p, q):
    r = Signal(bool(1))
    s = Signal(bool(1))
    inst1 = my_bundle(p, r)
    inst2 = or_gate(r, s, q)
    return inst1, inst2


@block
def adder(a, b, c):

    @instance
    def comb():
        while 1:
            yield a, b
            c.next = a + b

    return comb


@block
def ConstWire3(p, q):
    t = Signal(intbv(17)[5:])
    adder_inst = adder(p, t, q)
    return instances()


@block
def ConstWire_v(name, p, q):
    return setupCosimulation(**locals())


class TestConstWires(unittest.TestCase):

    def benchBool(self, ConstWire):

        p = Signal(bool(0))
        q = Signal(bool(0))
        q_v = Signal(bool(0))

        constwire_inst = ConstWire(p, q).convert(hdl='Verilog')
        constwire_v_inst = ConstWire_v(ConstWire.__name__, p, q_v)

        def stimulus():
            for dummy in range(100):
                p.next = randrange(2)
                yield delay(10)
                self.assertEqual(q, q_v)

        return stimulus(), constwire_inst, constwire_v_inst

    def testConstWire1(self):
        sim = self.benchBool(my_bundle)
        Simulation(sim).run()

    def testConstWire2(self):
        sim = self.benchBool(ConstWire2)
        Simulation(sim).run()

    def benchIntbv(self, ConstWire):

        p = Signal(intbv(0)[8:])
        q = Signal(intbv(0)[8:])
        q_v = Signal(intbv(0)[8:])

        constwire_inst = ConstWire(p, q).convert(hdl='Verilog')
        constwire_v_inst = ConstWire_v(ConstWire.__name__, p, q_v)

        def stimulus():
            for i in range(100):
                p.next = i
                yield delay(10)
                self.assertEqual(q, q_v)

        return stimulus(), constwire_inst, constwire_v_inst

    def testConstWire3(self):
        sim = self.benchIntbv(ConstWire3)
        Simulation(sim).run()

### tests of code ignore facility during translation ###


@block
def adderRef(a, b, c):

    @instance
    def comb():
        while 1:
            yield a, b
            c.next = a + b

    return comb


@block
def adderDebug(a, b, c):

    @instance
    def comb():
        while 1:
            yield a, b
            if __debug__:
                import string
            c.next = a + b

    return comb


def Ignorecode_v(name, a, b, c):
    return setupCosimulation(**locals())


class TestIgnoreCode(unittest.TestCase):

    def bench(self, adder):

        a = Signal(intbv(0)[8:])
        b = Signal(intbv(0)[8:])
        c = Signal(intbv(0)[9:])
        c_v = Signal(intbv(0)[9:])

        ignorecode_inst = adder(a, b, c).convert(hdl='Verilog')
        ignorecode_v_inst = Ignorecode_v(adder.__name__, a, b, c_v)

        def stimulus():
            for dummy in range(100):
                a.next = randrange(2 ** 8)
                b.next = randrange(2 ** 8)
                yield delay(10)
                self.assertEqual(c, c_v)

        return stimulus(), ignorecode_inst, ignorecode_v_inst

    def testAdderRef(self):
        sim = self.bench(adderRef)
        Simulation(sim).run()

    def testAdderDebug(self):
        sim = self.bench(adderDebug)
        Simulation(sim).run()


if __name__ == '__main__':
    unittest.main()



import os
path = os.path

import random
from random import randrange
random.seed(2)

from myhdl import (block, Signal, intbv, delay, instance,
                   always, always_comb, StopSimulation, conversion)

from myhdl import ConversionError

ACTIVE_LOW, INACTIVE_HIGH = 0, 1


@block
def incRef(count, enable, clock, reset, n):
    """ Incrementer with enable.

    count -- output
    enable -- control input, increment when 1
    clock -- clock input
    reset -- asynchronous reset input
    n -- counter max value
    """

    @instance
    def comb():
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                if enable:
                    count.next = (count + 1) % n

    return comb


@block
def incGen(count, enable, clock, reset, n):
    """ Generator with vhdl_code is not permitted """

    @instance
    def comb():
        incGen.vhdl_code = "Template string"
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                if enable:
                    count.next = (count + 1) % n

    return comb


@block
def inc(count, enable, clock, reset, n):
    """ Incrementer with enable.

    count -- output
    enable -- control input, increment when 1
    clock -- clock input
    reset -- asynchronous reset input
    n -- counter max value
    """

    @always(clock.posedge, reset.negedge)
    def incProcess():
        # make it fail in conversion
        import types
        if reset == ACTIVE_LOW:
            count.next = 0
        else:
            if enable:
                count.next = (count + 1) % n

    count.driven = "reg"

    inc.vhdl_code = \
"""
process ($clock, $reset) begin
    if ($reset = '0') then
        $count <= (others => '0');
    elsif rising_edge($clock) then
        if ($enable = '1') then
            $count <= ($count + 1) mod $n;
        end if;
    end if;
end process;
"""

    return incProcess


@block
def incErr(count, enable, clock, reset, n):

    @always(clock.posedge, reset.negedge)
    def incProcess():
        # make it fail in conversion
        import types
        if reset == ACTIVE_LOW:
            count.next = 0
        else:
            if enable:
                count.next = (count + 1) % n

    count.driven = "reg"

    incErr.vhdl_code = \
"""
always @(posedge $clock, negedge $reset) begin
    if ($reset == 0) begin
        $count <= 0;
    end
    else begin
        if ($enable) begin
            $count <= ($countq + 1) %% $n;
        end
    end
end
"""

    return incProcess


@block
def inc_comb(nextCount, count, n):

    @always_comb
    def comb():
        # make if fail in conversion
        import types
        nextCount.next = (count + 1) % n

    nextCount.driven = "wire"

    inc_comb.vhdl_code = \
"""
$nextCount <= ($count + 1) mod $n;
"""

    return comb


@block
def inc_seq(count, nextCount, enable, clock, reset):

    @always(clock.posedge, reset.negedge)
    def comb():
        # make if fail in conversion
        import types
        if reset == ACTIVE_LOW:
            count.next = 0
        else:
            if (enable):
                count.next = nextCount

    count.driven = True

    inc_seq.vhdl_code = \
"""
process ($clock, $reset) begin
    if ($reset = '0') then
        $count <= (others => '0');
    elsif rising_edge($clock) then
        if ($enable = '1') then
            $count <= $nextCount;
        end if;
    end if;
end process;
"""

    return comb


@block
def inc2(count, enable, clock, reset, n):

    nextCount = Signal(intbv(0, min=0, max=n))

    comb = inc_comb(nextCount, count, n)
    seq = inc_seq(count, nextCount, enable, clock, reset)

    return comb, seq


@block
def inc3(count, enable, clock, reset, n):
    inc2_inst = inc2(count, enable, clock, reset, n)
    return inc2_inst


class ClassIncrementer(object):

    @block
    def inc(self, count, enable, clock, reset, n):
        """ Incrementer with enable.

        count -- output
        enable -- control input, increment when 1
        clock -- clock input
        reset -- asynchronous reset input
        n -- counter max value
        """

        @always(clock.posedge, reset.negedge)
        def incProcess():
            # make it fail in conversion
            import types
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                if enable:
                    count.next = (count + 1) % n

        count.driven = "reg"

        self.inc.vhdl_code = \
    """
    process ($clock, $reset) begin
        if ($reset = '0') then
            $count <= (others => '0');
        elsif rising_edge($clock) then
            if ($enable = '1') then
                $count <= ($count + 1) mod $n;
            end if;
        end if;
    end process;
    """

        return incProcess


@block
def clockGen(clock):

    @instance
    def comb():
        clock.next = 1
        while 1:
            yield delay(10)
            clock.next = not clock

    return comb


NRTESTS = 1000

ENABLES = tuple([min(1, randrange(5)) for i in range(NRTESTS)])


@block
def stimulus(enable, clock, reset):

    @instance
    def comb():
        reset.next = INACTIVE_HIGH
        yield clock.negedge
        reset.next = ACTIVE_LOW
        yield clock.negedge
        reset.next = INACTIVE_HIGH
        for i in range(NRTESTS):
            enable.next = 1
            yield clock.negedge
        for i in range(NRTESTS):
            enable.next = ENABLES[i]
            yield clock.negedge
        raise StopSimulation

    return comb


@block
def check(count, enable, clock, reset, n):

    @instance
    def comb():
        expect = 0
        yield reset.posedge
        # assert count == expect
        print(count)
        while 1:
            yield clock.posedge
            if enable:
                expect = (expect + 1) % n
            yield delay(1)
            # print "%d count %s expect %s count_v %s" % (now(), count, expect, count_v)
            # assert count == expect
            print(count)

    return comb


@block
def customBench(inc):

    m = 8
    n = 2 ** m

    count = Signal(intbv(0)[m:])
    enable = Signal(bool(0))
    clock, reset = [Signal(bool(1)) for i in range(2)]

    inc_inst = inc(count, enable, clock, reset, n=n)
    clk_1 = clockGen(clock)
    st_1 = stimulus(enable, clock, reset)
    ch_1 = check(count, enable, clock, reset, n=n)

    return inc_inst, clk_1, st_1, ch_1


def testIncRef():
    assert conversion.verify(customBench(incRef)) == 0


def testInc():
    assert conversion.verify(customBench(inc)) == 0


def testInc2():
    assert conversion.verify(customBench(inc2)) == 0


def testInc3():
    assert conversion.verify(customBench(inc3)) == 0


def testIncGen():
    try:
        assert conversion.verify(customBench(incGen)) == 0
    except ConversionError:
        pass
    else:
        assert False


def testIncErr():
    try:
        assert conversion.verify(customBench(incErr)) == 0
    except ConversionError:
        pass
    else:
        assert False


def testMethodInc():
    incrementer = ClassIncrementer()
    assert conversion.verify(customBench(incrementer.inc)) == 0




'''
Created on 3 dec. 2024

@author: josy
'''

from myhdl import (HdlClass, Signal, intbv, block, always_seq, OpenPort, always_comb,
                   Constant)


class Counter(HdlClass):

    def __init__(self, RANGE, Clk, Reset, SClr, CntEn, Q=None, IsMax=None, WRAP_AROUND=False):
        '''
            RANGE: int : number of counts
            Clk: Signal(bool()): the domain clock
            Reset: ResetSignal(): its associated reset
            SClr: Signal(bool()): resets the count
            CntEn: Signal(bool()): advances count
            Q: Signal(intbv()[w:]): the actual count
        '''
        self.Clk = Clk
        self.Reset = Reset
        self.SClr = SClr
        self.CntEn = CntEn
        self.RANGE = RANGE
        self.Q = Q if Q is not None else Signal(intbv(0, 0, RANGE))
        self.IsMax = IsMax if IsMax is not None else Signal(bool(0))
        self.WRAP_AROUND = WRAP_AROUND

    @block(skipname=True)
    def hdl(self):
        if isinstance(self.Q, OpenPort):
            count = Signal(intbv(0, 0, self.RANGE))
        else:
            count = self.Q

        if self.WRAP_AROUND:

            @always_seq(self.Clk.posedge, reset=self.Reset)
            def upcounter():
                if self.SClr or self.CntEn:
                    if self.SClr:
                        count.next = 0
                    else:
                        if count < (self.RANGE - 1):
                            count.next = count + 1
                        else:
                            count.next = 0

        else:

            @always_seq(self.Clk.posedge, reset=self.Reset)
            def upcounter():
                if self.SClr or self.CntEn:
                    if self.SClr:
                        count.next = 0
                    else:
                        if count < (self.RANGE - 1):
                            count.next = count + 1

        @always_comb
        def mkismax():
            self.IsMax.next = (count == self.RANGE - 1)

        return self.hdlinstances()


class PwmCounter(HdlClass):

    def __init__(self, RANGE, Clk, Reset, PwmValue, PwmOut=None):
        self.RANGE = RANGE
        self.Clk = Clk
        self.Reset = Reset
        self.PwmValue = PwmValue
        self.PwmOut = PwmOut if PwmOut is not None else Signal(bool(0))

    @block
    def hdl(self):
        counter = Counter(self.RANGE, self.Clk, self.Reset, SClr=Constant(bool(0)), CntEn=Constant(bool(1)), IsMax=OpenPort(), WRAP_AROUND=True)

        @always_seq(self.Clk.posedge, reset=self.Reset)
        def pwm():
            if counter.Q >= self.PwmValue:
                self.PwmOut.next = 0
            else:
                self.PwmOut.next = 1

        return self.hdlinstances()


if __name__ == '__main__':

    from myhdl import ResetSignal, instance, delay, StopSimulation, instances

    class XYMotors(HdlClass):

        def __init__(self, PWMCOUNT, Clk, Reset, XSpeed, YSpeed, XDrive, YDrive):
            # mess up the order of the signals trying to confuse the convertor
            # as we have to tweak things a bit to achieve 'direct' conversion of
            # HdlClass objects
            self.Reset = Reset
            self.XDrive = XDrive
            self.PWMCOUNT = PWMCOUNT
            self.Clk = Clk
            self.XSpeed = XSpeed
            self.YSpeed = YSpeed
            self.YDrive = YDrive

        @block(skipname=True)
        def hdl(self):
            xmotor = PwmCounter(self.PWMCOUNT, self.Clk, self.Reset, self.XSpeed, self.XDrive)
            ymotor = PwmCounter(self.PWMCOUNT, self.Clk, self.Reset, self.YSpeed, self.YDrive)

            return self.hdlinstances()

    # create a minimal test-bench to test the .vcd generation
    # as we want/have to weed out the `None` - because of an @block(skipname=True)
    # which add an unnecessary indentation level in the waveform which absolutely looks ugly
    @block
    def tb_xymotors():
        PWMCOUNT = 100
        Clk = Signal(bool(0))
        Reset = ResetSignal(0, 1, False)
        XSpeed = Signal(intbv(0, 0, PWMCOUNT))
        YSpeed = Signal(intbv(0, 0, PWMCOUNT))
        XDrive = Signal(bool(0))
        YDrive = Signal(bool(0))

        dft = XYMotors(PWMCOUNT, Clk, Reset, XSpeed, YSpeed, XDrive, YDrive)
        dfthdl = dft.hdl()
        dfthdl.name = 'XYMotors'

        tCK = 10

        @instance
        def genclkreset():
            Reset.next = 1
            for dummy in range(3):
                Clk.next = 1
                yield delay(tCK // 2)
                Clk.next = 0
                yield delay(tCK - tCK // 2)

            Clk.next = 1
            yield delay(tCK // 2)
            Clk.next = 0
            Reset.next = 0
            yield delay(tCK - tCK // 2)
            while True:
                Clk.next = 1
                yield delay(tCK // 2)
                Clk.next = 0
                yield delay(tCK - tCK // 2)

        @instance
        def stimulus():
            for dummy in range(10):
                yield Clk.posedge

            raise StopSimulation

        return instances()

    if 0:
        dft = tb_xymotors()
        dft.config_sim(trace=True, timescale='1ps', tracebackup=False)
        dft.run_sim()

    def convert():
        # try converting
        # note they will appear in this order in the entity/module declaration; why?
        XDrive = Signal(bool(0))
        PWMCOUNT = 100
        Reset = ResetSignal(0, 1, False)
        Clk = Signal(bool(0))
        XSpeed = Signal(intbv(0, 0, PWMCOUNT))
        YSpeed = Signal(intbv(0, 0, PWMCOUNT))
        YDrive = Signal(bool(0))

        if 0:
            ''' looks like we have to live with writing a wrapper '''

            # a local written-out wrapper works fine
            @block(skipname=True)
            def wrapper(PWMCOUNT, Clk, Reset, XSpeed, YSpeed, XDrive, YDrive):
                dfc = XYMotors(PWMCOUNT, Clk, Reset, XSpeed, YSpeed, XDrive, YDrive)
                dfchdl = dfc.hdl()
                # ! DO NOT override the name
                # e.g.: dfchdl.name = 'blabla' will prefix all names with 'blabla'
                return dfchdl

            dfc = wrapper(PWMCOUNT, Clk, Reset, XSpeed, YSpeed, XDrive, YDrive)
            dfc.convert(hdl='VHDL', name='XYMotors')
            dfc.convert(hdl='Verilog', name='XYMotors')

        else:
            if 0:

                @block
                def wrapper(hdlclass, *args):
                    return hdlclass(*args).hdl()

                # this raises an IndexError in _analyze.py
                # beacuse the '*args' in wrapper disappear into nowhere?
                dfc = wrapper(XYMotors, PWMCOUNT, Clk, Reset, XSpeed, YSpeed, XDrive, YDrive)
                for key, value in vars(dfc).items():
                    print(key, value)
                print()
                print(f'{dfc.sigdict=}')
                dfc.convert(hdl='VHDL', name='XYMotors')
            else:
                # doing direct conversion from the class instance itself
                # this is quite necessary for hierarchical conversion
                dfc = XYMotors(PWMCOUNT, Clk, Reset, XSpeed, YSpeed, XDrive, YDrive)
                dfc.convert(hdl='VHDL', name='XYMotors')
                dfc.convert(hdl='Verilog', name='XYMotors')

    convert()




from myhdl import (Signal, intbv, concat)


def unsigned(width, value=0, cls=intbv):
    """Create an unsigned signal based on a bitvector with the
    specified width and initial value.
    """
    return Signal(cls(value, 0, 2 ** width))


def signed(width, value=0, cls=intbv):
    """Create an signed signal based on a bitvector with the
    specified width and initial value.
    """
    return Signal(cls(value, -2 ** (width - 1), 2 ** (width - 1)))


a = unsigned(4, 8)
b = signed(28, -3)


# print "%08X" % myhdl.concat(a, b)
# print hex(myhdl.concat(a, b))
def test_issue_10():
    assert concat(a, b) == 0x8ffffffd



from myhdl import (block, Signal, intbv, instance, delay)

from myhdl import ConversionError
from myhdl.conversion._misc import _error


@block
def issue_122(dout, i):

    d = i * 10 + 1

    @instance
    def write():
        dout[i].next = i
        yield delay(d)
        print(int(dout[i]))

    if i == 0:
        return write
    else:
        inst = issue_122(dout, i - 1)
        return write, inst


def tb_issue_122():
    n = 7
    dout = [Signal(intbv(0, min=0, max=n + 1)) for __ in range(n + 1)]
    inst = issue_122(dout, n)
    return inst


def test_issue_122():
    try:
        tb_issue_122().verify_convert()
    except ConversionError as e:
        assert e.kind == _error.ListAsPort
    else:
        assert False



from random import randrange

from myhdl import (block, Signal, enum, intbv, modbv, delay, always_comb, always_seq,
                   always, instance, instances, StopSimulation, toVHDL, toVerilog,
                   ResetSignal, conversion)


@block
def initial_value_enum_bench(initial_val, **kwargs):
    clk = Signal(bool(0))
    sig = Signal(initial_val)
    states = kwargs['states']
    N = 20

    if initial_val in (states.a, states.c):
        valid_states = (states.a, states.c)

    else:
        valid_states = (states.b, states.d)

    @instance
    def clkgen():

        clk.next = 0
        for dummy in range(N):
            yield delay(10)
            clk.next = not clk

        raise StopSimulation()

    @always(clk.posedge)
    def state_walker():
        if sig == states.a:
            sig.next = states.c
            print('a')
        elif sig == states.c:
            sig.next = states.a
            print('c')
        elif sig == states.b:
            sig.next = states.d
            print('b')
        elif sig == states.d:
            sig.next = states.b
            print('d')

        if __debug__:
            assert sig in valid_states

    return state_walker, clkgen


@block
def bool_writer(signal, clk):

    @always(clk.posedge)
    def writer():
        print(int(signal))

    return writer


@block
def int_writer(signal, clk):

    @always(clk.posedge)
    def writer():
        print(signal)

    return writer


@block
def initial_value_bench(initial_val, **kwargs):

    clk = Signal(bool(0))

    input_signal = Signal(initial_val)

    if 'change_input_signal' in kwargs.keys():

        change_input_signal = kwargs['change_input_signal']
    else:
        change_input_signal = False

    if change_input_signal:
        # Make sure it doesn't overflow when changing
        if initial_val > 0:
            input_signal.val[:] = initial_val - 1
        else:
            input_signal.val[:] = initial_val + 1

    if isinstance(initial_val, bool):
        output_signal = Signal(not initial_val)
        update_val = not initial_val

    else:
        output_signal = Signal(
            intbv(0, min=initial_val.min, max=initial_val.max))
        update_val = 0

    expected_output = input_signal._init

    N = 10
    first = [True]

    @instance
    def clkgen():

        clk.next = 0
        for dummy in range(N):
            yield delay(10)
            clk.next = not clk

        raise StopSimulation()

    @always_comb
    def output_driver():
        output_signal.next = input_signal

    @always(clk.posedge)
    def drive_and_check():

        input_signal.next = update_val

        if __debug__:
            if first[0]:
                assert output_signal == expected_output
                first[0] = False
            else:
                assert output_signal == update_val

    if isinstance(initial_val, bool):
        output_writer = bool_writer(output_signal, clk)

    else:
        output_writer = int_writer(output_signal, clk)

    return clkgen, output_driver, drive_and_check, output_writer


@block
def canonical_list_writer(output_signal_list, clk):

    signal_list_length = len(output_signal_list)

    @always(clk.posedge)
    def list_writer():
        for i in range(signal_list_length):
            print(str(output_signal_list[i]._val))

    canonical_list_writer.verilog_code = '''
always @(posedge $clk) begin: INITIAL_VALUE_LIST_BENCH_CANONICAL_LIST_WRITER_0_LIST_WRITER
    integer i;
    for (i=0; i<10; i=i+1) begin
        $$write("%h", output_signal_list[i]);
        $$write("\\n");
    end
end
'''
    canonical_list_writer.vhdl_code = '''
INITIAL_VALUE_BENCH_OUTPUT_WRITER: process ($clk) is
    variable L: line;
begin
    if rising_edge($clk) then
        for i in 0 to $signal_list_length-1 loop
            write(L, to_hstring(unsigned(output_signal_list(i))));
            writeline(output, L);
        end loop;
    end if;
end process INITIAL_VALUE_BENCH_OUTPUT_WRITER;
'''
    return list_writer


@block
def bool_list_writer(output_signal_list, clk):

    signal_list_length = len(output_signal_list)

    @always(clk.posedge)
    def list_writer():
        for i in range(signal_list_length):
            print(int(output_signal_list[i]))

    return list_writer


@block
def initial_value_bool_list_bench(initial_vals, **kwargs):
    clk = Signal(bool(0))

    input_signal_list = [Signal(initial_val) for initial_val in initial_vals]

    output_signal_list = [
        Signal(not initial_val) for initial_val in initial_vals]

    update_val = int(not initial_vals[0])

    expected_output = [
        bool(each_input._init) for each_input in input_signal_list]

    N = 10
    first = [True]

    signal_list_length = len(initial_vals)

    @instance
    def clkgen():

        clk.next = 0
        for dummy in range(N):
            yield delay(10)
            clk.next = not clk

        raise StopSimulation()

    @always_comb
    def output_driver():
        for i in range(signal_list_length):
            output_signal_list[i].next = input_signal_list[i]

    @always(clk.posedge)
    def drive_and_check():

        for i in range(signal_list_length):
            input_signal_list[i].next = update_val

        if __debug__:
            if first[0]:
                for i in range(signal_list_length):
                    assert output_signal_list[i] == expected_output[i]
                first[0] = False
            else:
                for i in range(signal_list_length):
                    assert output_signal_list[i] == update_val

    output_writer = bool_list_writer(output_signal_list, clk)

    return clkgen, output_driver, drive_and_check, output_writer


@block
def assign_output(input_signal, output_signal):

    @always_comb
    def assignment():
        output_signal.next = input_signal

    return assignment


@block
def initial_value_list_bench(initial_vals, **kwargs):
    clk = Signal(bool(0))

    input_signal_list = [Signal(initial_val) for initial_val in initial_vals]

    if len(initial_vals[0]) == 1:

        output_signal_list = [
            Signal(intbv(not initial_val, min=0, max=2)) for
            initial_val in initial_vals]
        update_val = int(not initial_vals[0])
    else:
        output_signal_list = [
            Signal(intbv(0, min=initial_val.min, max=initial_val.max)) for
            initial_val in initial_vals]
        update_val = 0

    expected_output = [each_input._init for each_input in input_signal_list]

    N = 10
    first = [True]

    signal_list_length = len(initial_vals)

    @instance
    def clkgen():

        clk.next = 0
        for dummy in range(N):
            yield delay(10)
            clk.next = not clk

        raise StopSimulation()

    # We assign each of the output drivers independently.
    # This forces the output to be a wire (where appropriate) so we can
    # check this type is handled properly too.
    output_drivers = []
    for input_signal, output_signal in zip(
        input_signal_list, output_signal_list):

        output_drivers.append(assign_output(input_signal, output_signal))

    @always(clk.posedge)
    def drive_and_check():

        for i in range(signal_list_length):
            input_signal_list[i].next = update_val

        if __debug__:
            if first[0]:
                for i in range(signal_list_length):
                    assert output_signal_list[i] == expected_output[i]
                first[0] = False
            else:
                for i in range(signal_list_length):
                    assert output_signal_list[i] == update_val

    output_writer = canonical_list_writer(output_signal_list, clk)

    return clkgen, output_drivers, drive_and_check, output_writer


@block
def initial_value_mem_convert_bench():

    clock = Signal(bool(0))
    reset = ResetSignal(0, active=0, isasync=True)
    wr = Signal(bool(0))
    wrd = Signal(intbv(0, min=0, max=32))
    rdd = Signal(intbv(0, min=0, max=32))
    addr = Signal(intbv(0, min=0, max=16))

    inst = memory(clock, reset, wr, wrd, rdd, addr)

    tCKhalf = 10 // 2
    tReset = int (10 * 3.5)

    @instance
    def genclk():
        while True:
            clock.next = not clock
            yield delay(tCKhalf)

    @instance
    def genreset():
        reset.next = 0
        yield delay(tReset)
        reset.next = 1

    return instances()


@block
def memory(clock, reset, wr, wrd, rdd, addr):

    mem = [Signal(intbv(0, min=wrd.min, max=wrd.max))
           for __ in range(addr.max)]

    inst_init = memory_init(mem)

    @always_seq(clock.posedge, reset=reset)
    def beh_mem():
        rdd.next = mem[addr]
        if wr:
            mem[addr].next = wrd

    return inst_init, beh_mem


@block
def memory_init(mem):
    mem_size = len(mem)
    init_values = tuple([int(ss.val) for ss in mem])

    with open("init_file.hex", 'w') as fp:
        for ii in range(mem_size):
            fp.write("CE \n")

    @instance
    def beh_init():
        for ii in range(mem_size):
            mem[ii].next = init_values[ii]
        yield delay(10)

    return beh_init


memory_init.verilog_code = """
    initial begin
        $$readmemh("init_file.hex", $mem, $mem_size);
    end
"""


def runner(initial_val, tb=initial_value_bench, **kwargs):
    pre_toVerilog_initial_values = toVerilog.initial_values
    pre_toVHDL_initial_values = toVHDL.initial_values

    toVerilog.initial_values = True
    toVHDL.initial_values = True

    try:
        assert conversion.verify(tb(initial_val, **kwargs)) == 0

    finally:
        toVerilog.initial_values = pre_toVerilog_initial_values
        toVHDL.initial_values = pre_toVHDL_initial_values


def test_unsigned():
    '''The correct initial value should be used for unsigned type signal.
    '''
    min_val = 0
    max_val = 34
    initial_val = intbv(
        randrange(min_val, max_val), min=min_val, max=max_val)

    runner(initial_val)


def test_signed():
    '''The correct initial value should be used for signed type signal.
    '''
    min_val = -12
    max_val = 4

    initial_val = intbv(
        randrange(min_val, max_val), min=min_val, max=max_val)

    runner(initial_val)


def test_bool():
    '''The correct initial value should be used for bool type signal.
    '''
    initial_val = bool(randrange(0, 2))
    runner(initial_val)


def test_modbv():
    '''The correct initial value should be used for modbv type signal.
    '''

    initial_val = modbv(randrange(0, 2 ** 10))[10:]

    runner(initial_val)


def test_enum():
    '''The correct initial value should be used for enum type signals.
    '''
    states = enum('a', 'b', 'c', 'd')
    val1 = states.c
    val2 = states.b

    runner(val1, tb=initial_value_enum_bench, states=states)
    runner(val2, tb=initial_value_enum_bench, states=states)


def test_long_signals():
    '''The correct initial value should work with wide bitwidths (i.e. >32)
    '''
    min_val = -(2 ** 71)
    max_val = 2 ** 71 - 1
    initial_val = intbv(
        randrange(min_val, max_val), min=min_val, max=max_val)

    runner(initial_val)


def test_single_length_signals():
    '''The correct initial value should be used for a single length signal
    '''
    initial_val = intbv(0, min=0, max=2)

    runner(initial_val)


def test_unsigned_list():
    '''The correct initial value should be used for unsigned type signal lists
    '''
    min_val = 0
    max_val = 34
    initial_vals = [intbv(randrange(min_val, max_val), min=min_val, max=max_val)
                            for __ in range(10)]

    runner(initial_vals, tb=initial_value_list_bench)

    # All the same case
    initial_vals = [
        intbv(randrange(min_val, max_val), min=min_val, max=max_val)] * 10
    runner(initial_vals, tb=initial_value_list_bench)


def test_signed_list():
    '''The correct initial value should be used for signed type signal lists
    '''
    min_val = -12
    max_val = 4

    initial_vals = [intbv(randrange(min_val, max_val), min=min_val, max=max_val)
        for __ in range(10)]

    runner(initial_vals, tb=initial_value_list_bench)

    # All the same case
    initial_vals = [intbv(
        randrange(min_val, max_val), min=min_val, max=max_val)] * 10

    runner(initial_vals, tb=initial_value_list_bench)


def test_modbv_list():
    '''The correct initial value should be used for modbv type signal lists
    '''

    initial_vals = [modbv(randrange(0, 2 ** 10))[10:] for __ in range(10)]

    runner(initial_vals, tb=initial_value_list_bench)

    # All the same case
    initial_vals = [modbv(randrange(0, 2 ** 10))[10:]] * 10
    runner(initial_vals, tb=initial_value_list_bench)


def test_long_signals_list():
    '''The correct initial value should work with wide bitwidths (i.e. >32)
    signal lists
    '''
    min_val = -(2 ** 71)
    max_val = 2 ** 71 - 1
    initial_vals = [intbv(randrange(min_val, max_val), min=min_val, max=max_val)
        for __ in range(10)]

    runner(initial_vals, tb=initial_value_list_bench)

    # All the same case
    initial_vals = [intbv(2 ** 65 - 50, min=min_val, max=max_val)] * 10
    runner(initial_vals, tb=initial_value_list_bench)


def test_bool_signals_list():
    '''The correct initial value should be used for a boolean type signal lists
    '''
    initial_vals = [False for __ in range(10)]

    runner(initial_vals, tb=initial_value_bool_list_bench)

    initial_vals = [False] * 10
    runner(initial_vals, tb=initial_value_bool_list_bench)


def test_init_used():
    '''It should be the _init attribute that is used for initialisation

    It should not be the current value, which should be ignored.
    '''
    min_val = -34
    max_val = 15
    initial_val = intbv(
        randrange(min_val, max_val), min=min_val, max=max_val)

    runner(initial_val, change_input_signal=True)


def test_memory_convert():
    inst = initial_value_mem_convert_bench()

    # TODO: this needs to be converted to use the `block` convert
    #       only and not modify the `toV*` but this will require
    #       changes to `conversion.verify` and `conversion.analyze`
    #       or a `config_conversion` function add to the `Block`.
    pre_xiv = toVerilog.initial_values
    pre_viv = toVHDL.initial_values

    # not using the runner, this test is setup for analyze only
    toVerilog.initial_values = True
    toVHDL.initial_values = True

    try:
        # assert conversion.analyze(inst) == 0
        assert inst.analyze_convert() == 0

    finally:
        toVerilog.initial_values = pre_xiv
        toVHDL.initial_values = pre_viv


@block
def init_reset_tb():

    clk = Signal(bool(0))
    reset = ResetSignal(0, active=1, isasync=False)

    s_large = Signal(intbv(0xc0000000)[32:])
    s_small = Signal(intbv(0xc)[32:])

    @instance
    def clkgen():

        clk.next = 0
        for dummy in range(10):
            yield delay(10)
            clk.next = not clk

        raise StopSimulation()

    @instance
    def raise_reset():
        yield clk.posedge
        reset.next = 1
        yield clk.posedge
        reset.next = 0

    @always_seq(clk.posedge, reset=reset)
    def seq():

        print(s_large)
        print(s_small)
        s_large.next = s_large + 1
        s_small.next = s_small + 1

    return instances()


def test_init_reset():
    """ Test assignment of initial values of signals used in an always_seq block with a reset signal
        Because the _convertInitVal in _toVHDL.py does special handling depending on the init value
        the test takes this into account.
    """

    inst = init_reset_tb()
    assert conversion.verify(inst, initial_values=True) == 0

# def test_init_used_list():
#    '''It should be the _init attribute of each element in the list
#    that is used for initialisation
#
#    It should not be the current value, which should be ignored.
#    '''
#    min_val = -34
#    max_val = 15
#    initial_val = [intbv(
#        randrange(min_val, max_val), min=min_val, max=max_val)
#        for each in range(10)]
#
#    list_runner(initial_val, change_input_signal=True)


if __name__ == "__main__":
    test_signed_list()




from random import randrange

from myhdl import (block, Signal, intbv, delay, instance, conversion)


@block
def NumassBench():

    p = Signal(intbv(1)[8:])
    q = Signal(intbv(1)[40:])
    r = Signal(intbv(1, min=-256, max=256))
    s = Signal(intbv(1, min=-2 ** 40, max=2 ** 40))
    PBIGINT = randrange(2 ** 34, 2 ** 40)
    NBIGINT = -randrange(2 ** 34, 2 ** 40)

    @instance
    def check():
        p.next = 0
        q.next = 0
        r.next = 0
        s.next = 0
        yield delay(10)
        print("%d %d %d %d" % (p, q, r, s))
        p.next = 1
        q.next = 1
        r.next = 1
        s.next = 1
        yield delay(10)
        print("%d %d %d %d" % (p, q, r, s))
        p.next = 2
        q.next = 2
        r.next = -2
        s.next = -2
        yield delay(10)
        print("%d %d %d %d" % (p, q, r, s))
        p.next = 255
        q.next = 246836311517
        r.next = 255
        s.next = -246836311517
        yield delay(10)
        print("%d %d %d %d %d %d" % (p, q[40:20], q[20:0], r , s[41:20], s[20:0]))
        p.next = 254
        q.next = PBIGINT
        r.next = -256
        s.next = NBIGINT
        yield delay(10)
        print("%d %d %d %d %d %d" % (p, q[40:20], q[20:0], r , s[41:20], s[20:0]))

    return check


def test_numass():
    assert conversion.verify(NumassBench()) == 0



from myhdl import (block, Signal, intbv, always_comb)


@block
def constants(v, u, x, y, z, a):

    b = Signal(bool(0))
    c = Signal(bool(1))
    d = Signal(intbv(5)[8:])
    e = Signal(intbv(4, min=-3, max=9))

    @always_comb
    def comb():
        u.next = d
        v.next = e
        x.next = b
        y.next = c
        z.next = a

    return comb


x, y, z, a = [Signal(bool(0)) for i in range(4)]
u = Signal(intbv(0)[8:])
v = Signal(intbv(0, min=-3, max=9))


def test_constants():
    assert constants(v, u, x, y, z, a).analyze_convert() == 0



import myhdl
from myhdl import *


def timer_sig(flag, clock, reset, MAXVAL):

    count = Signal(intbv(0, min=0, max=MAXVAL + 1))

    @always (clock.posedge, reset.posedge)
    def synch():
        if reset == 1:
            count.next = 0
        else:
            flag.next = 0
            if count == MAXVAL:
                flag.next = 1
                count.next = 0
            else:
                count.next = count + 1

    return synch


def timer_var(flag, clock, reset, MAXVAL):

    @instance
    def comb():
        count = intbv(0, min=0, max=MAXVAL + 1)
        while True:
            yield clock.posedge, reset.posedge
            if reset == 1:
                count[:] = 0
            else:
                flag.next = 0
                if count == MAXVAL:
                    flag.next = 1
                    count[:] = 0
                else:
                    count += 1

    return comb




from myhdl import Signal, ResetSignal, modbv

from gray_inc_reg import gray_inc_reg

def convert_gray_inc_reg(hdl, width=8):
    graycnt = Signal(modbv(0)[width:])
    enable = Signal(bool())
    clock = Signal(bool())
    reset = ResetSignal(0, active=0, isasync=True)

    inst = gray_inc_reg(graycnt, enable, clock, reset, width)
    inst.convert(hdl)

convert_gray_inc_reg(hdl='Verilog')
convert_gray_inc_reg(hdl='VHDL')



#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2008 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Run the unit tests for instance """
from myhdl import (InstanceError, Signal, instance)
from myhdl._instance import _error
from helpers import raises_kind

# random.seed(3) # random, but deterministic

QUIET = 1


def g():
    pass


x = Signal(0)


class TestInstanceCompilation:

    def testArgIsFunction(self):
        h = 5
        with raises_kind(InstanceError, _error.ArgType):
            instance(h)

    def testArgIsGeneratorFunction(self):
        with raises_kind(InstanceError, _error.ArgType):

            @instance
            def h():
                return None

    def testArgHasNoArgs(self):
        with raises_kind(InstanceError, _error.NrOfArgs):

            @instance
            def h(n):
                yield n



import os
path = os.path
import unittest
from unittest import TestCase

from myhdl import enum, block, Signal, intbv, always, instance, delay, StopSimulation

from .util import setupCosimulation

# SEARCH, CONFIRM, SYNC = range(3)
ACTIVE_LOW = 0
FRAME_SIZE = 8
t_State_b = enum('SEARCH', 'CONFIRM', 'SYNC')
t_State_oh = enum('SEARCH', 'CONFIRM', 'SYNC', encoding="one_hot")
t_State_oc = enum('SEARCH', 'CONFIRM', 'SYNC', encoding="one_cold")


@block
def FramerCtrl(SOF, state, syncFlag, clk, reset_n, t_State):

    """ Framing control FSM.

    SOF -- start-of-frame output bit
    state -- FramerState output
    syncFlag -- sync pattern found indication input
    clk -- clock input
    reset_n -- active low reset

    """

    index = Signal(intbv(0)[8:])  # position in frame

    @always(clk.posedge, reset_n.negedge)
    def FSM():
        if reset_n == ACTIVE_LOW:
            SOF.next = 0
            index.next = 0
            state.next = t_State.SEARCH
        else:
            index.next = (index + 1) % FRAME_SIZE
            SOF.next = 0
            if state == t_State.SEARCH:
                index.next = 1
                if syncFlag:
                    state.next = t_State.CONFIRM
            elif state == t_State.CONFIRM:
                if index == 0:
                    if syncFlag:
                        state.next = t_State.SYNC
                    else:
                        state.next = t_State.SEARCH
            elif state == t_State.SYNC:
                if index == 0:
                    if not syncFlag:
                        state.next = t_State.SEARCH
                SOF.next = (index == FRAME_SIZE - 1)
            else:
                raise ValueError("Undefined state")

    return FSM


@block
def FramerCtrl_alt(SOF, state, syncFlag, clk, reset_n, t_State):

    """ Framing control FSM.

    SOF -- start-of-frame output bit
    state -- FramerState output
    syncFlag -- sync pattern found indication input
    clk -- clock input
    reset_n -- active low reset

    """

    @instance
    def FSM():
        index = intbv(0)[8:]  # position in frame
        state_var = t_State.SEARCH
        while 1:
            yield clk.posedge, reset_n.negedge
            if reset_n == ACTIVE_LOW:
                SOF.next = 0
                index[:] = 0
                state_var = t_State.SEARCH
                state.next = t_State.SEARCH
            else:
                SOF_var = 0
                if state == t_State.SEARCH:
                    index[:] = 0
                    if syncFlag:
                        state_var = t_State.CONFIRM
                elif state == t_State.CONFIRM:
                    if index == 0:
                        if syncFlag:
                            state_var = t_State.SYNC
                        else:
                            state_var = t_State.SEARCH
                elif state == t_State.SYNC:
                    if index == 0:
                        if not syncFlag:
                            state_var = t_State.SEARCH
                    SOF_var = (index == FRAME_SIZE - 1)
                else:
                    raise ValueError("Undefined state")
                index[:] = (index + 1) % FRAME_SIZE
                SOF.next = SOF_var
                state.next = state_var

    return FSM


@block
def FramerCtrl_ref(SOF, state, syncFlag, clk, reset_n, t_State):

    """ Framing control FSM.

    SOF -- start-of-frame output bit
    state -- FramerState output
    syncFlag -- sync pattern found indication input
    clk -- clock input
    reset_n -- active low reset

    """

    @instance
    def comb():
        index = intbv(0, min=0, max=8)  # position in frame
        while 1:
            yield clk.posedge, reset_n.negedge
            if reset_n == ACTIVE_LOW:
                SOF.next = 0
                index[:] = 0
                state.next = t_State.SEARCH
            else:
                SOF.next = 0
                if state == t_State.SEARCH:
                    index[:] = 0
                    if syncFlag:
                        state.next = t_State.CONFIRM
                elif state == t_State.CONFIRM:
                    if index == 0:
                        if syncFlag:
                            state.next = t_State.SYNC
                        else:
                            state.next = t_State.SEARCH
                elif state == t_State.SYNC:
                    if index == 0:
                        if not syncFlag:
                            state.next = t_State.SEARCH
                    SOF.next = (index == FRAME_SIZE - 1)
                else:
                    raise ValueError("Undefined state")
                index[:] = (index + 1) % FRAME_SIZE

    return comb


@block
def FramerCtrl_v(name, SOF, state, syncFlag, clk, reset_n):
    return setupCosimulation(**locals())


class FramerCtrlTest(TestCase):

    @block
    def bench(self, FramerCtrl, t_State):

        SOF = Signal(bool(0))
        SOF_v = Signal(bool(0))
        syncFlag = Signal(bool(0))
        clk = Signal(bool(0))
        reset_n = Signal(bool(1))
        state = Signal(t_State.SEARCH)
        state_v = Signal(intbv(0)[8:])

        framerctrl_ref_inst = FramerCtrl_ref(SOF, state, syncFlag, clk, reset_n, t_State)
        framerctrl_inst = FramerCtrl(SOF, state, syncFlag, clk, reset_n, t_State).convert()
        framerctrl_v_inst = FramerCtrl_v(FramerCtrl.__name__,
                                         SOF_v, state_v, syncFlag, clk, reset_n)

        @instance
        def clkgen():
            reset_n.next = 1
            yield delay(10)
            reset_n.next = 0
            yield delay(10)
            reset_n.next = 1
            yield delay(10)
            while 1:
                yield delay(10)
                clk.next = not clk

        @instance
        def stimulus():
            for dummy in range(3):
                yield clk.posedge
            for n in (12, 8, 8, 4, 11, 8, 8, 7, 6, 8, 8):
                syncFlag.next = 1
                yield clk.posedge
                syncFlag.next = 0
                for dummy in range(n - 1):
                    yield clk.posedge
            raise StopSimulation

        @instance
        def check():
            while 1:
                yield clk.negedge
                self.assertEqual(SOF, SOF_v)
                self.assertEqual(eval(hex(state)), eval(hex(state_v)))
                # print "MyHDL: %s %s" % (SOF, hex(state))
                # print "Verilog: %s %s" % (SOF_v, hex(state_v))

        return framerctrl_ref_inst, framerctrl_v_inst, clkgen, stimulus, check

    def testRef(self):
        for t_State in (t_State_b, t_State_oc, t_State_oh):
            tb_fsm = self.bench(FramerCtrl_ref, t_State)
            tb_fsm.run_sim()

    def testAlt(self):
        for t_State in (t_State_b, t_State_oc, t_State_oh):
            tb_fsm = self.bench(FramerCtrl_alt, t_State)
            tb_fsm.run_sim()

    def testDoc(self):
        tb_fsm = self.bench(FramerCtrl, t_State_oh)
        tb_fsm.run_sim()


if __name__ == '__main__':
    unittest.main()



'''
Created on 19 okt. 2023

@author: josy
'''

from myhdl import (block, Signal, intbv, always_comb, always_seq, instances,
                   OpenPort)


@block
def contrived_dff(Clk, D, Q, OE):

    @always_seq(Clk.posedge, reset=None)
    def cdff():
        Q.next = D
        OE.next = Q and D

    return instances()


@block
def contrived_dff2(Clk, D, Q, OE):

    @always_seq(Clk.posedge, reset=None)
    def sdff():
        Q.next = D

    @always_comb
    def cdff():
        OE.next = Q and D

    return instances()


@block
def wrapper(Clk, D, Q):
    return contrived_dff(Clk, D, Q, OE=OpenPort())


@block
def wrapper2(Clk, D, Q):
    return contrived_dff2(Clk, D, Q, OE=OpenPort())


def test_contrived_dff2():
        Clk = Signal(bool(0))
        D = Signal(bool(0))
        Q = Signal(bool(0))
        OE = Signal(bool(0))

        assert contrived_dff(Clk, D, Q, OE).analyze_convert() == 0


def test_wrapper():
        Clk = Signal(bool(0))
        D = Signal(bool(0))
        Q = Signal(bool(0))

        assert wrapper(Clk, D, Q).analyze_convert() == 0


def test_wrapper2():
        Clk = Signal(bool(0))
        D = Signal(bool(0))
        Q = Signal(bool(0))

        assert wrapper2(Clk, D, Q).analyze_convert() == 0


if __name__ == '__main__':

    def convert():
        Clk = Signal(bool(0))
        D = Signal(bool(0))
        Q = Signal(bool(0))
        OE = Signal(bool(0))

        dfc = contrived_dff(Clk, D, Q, OE)
        dfc.convert(hdl='VHDL')
        dfc.convert(hdl='Verilog')

        dfc2 = wrapper(Clk, D, Q)
        dfc2.convert(hdl='VHDL')
        dfc2.convert(hdl='Verilog')

        dfc3 = wrapper2(Clk, D, Q)
        dfc3.convert(hdl='VHDL')
        dfc3.convert(hdl='Verilog')

    convert()

    @block
    def dff(Clk, D, Q, Q_n):

        @always_seq(Clk.posedge, reset=None)
        def sdff():
            Q.next = D
            Q_n.next = not D

        return instances()

    @block
    def wrapper_dff(Clk, D, Q):
        return dff(Clk, D, Q, Q_n=OpenPort())

    Clk = Signal(bool(0))
    D = Signal(bool(0))
    Q = Signal(bool(0))

    dfc = wrapper_dff(Clk, D, Q)
    dfc.convert(hdl='Verilog')
    dfc.convert(hdl='VHDL')




import random
from myhdl import block, instance, Signal, intbv, delay
from mux import mux

random.seed(5)
randrange = random.randrange

@block
def test_mux():

    z, a, b, sel = [Signal(intbv(0)) for i in range(4)]

    mux_1 = mux(z, a, b, sel)

    @instance
    def stimulus():
        print("z a b sel")
        for i in range(12):
            a.next, b.next, sel.next = randrange(8), randrange(8), randrange(2)
            yield delay(10)
            print("%s %s %s %s" % (z, a, b, sel))

    return mux_1, stimulus

tb = test_mux()
tb.run_sim()



import os, sys
path = os.path
import subprocess
from glob import glob
from myhdl import Cosimulation


# Icarus
def setupCosimulationIcarus(**kwargs):
    name = kwargs['name']
    objfile = "%s.o" % name
    if path.exists(objfile):
        os.remove(objfile)
    analyze_cmd = ['iverilog', '-o', objfile, '%s.v' % name, 'tb_%s.v' % name]
    subprocess.call(analyze_cmd)
    vpifile = "myhdl"
    vpifiles = glob("**/myhdl.vpi", recursive=True)
    if 1 == len(vpifiles):
        vpifile = vpifiles[0]
    elif sys.platform != "win32":
        vpifile = "../../../../cosimulation/icarus/myhdl.vpi"
    simulate_cmd = ['vvp', '-m', vpifile, objfile]
    return Cosimulation(simulate_cmd, **kwargs)


# cver
def setupCosimulationCver(**kwargs):
    name = kwargs['name']
    cmd = "cver -q +loadvpi=../../../../cosimulation/cver/myhdl_vpi:vpi_compat_bootstrap " + \
          "%s.v tb_%s.v " % (name, name)
    return Cosimulation(cmd, **kwargs)


def verilogCompileIcarus(name):
    objfile = "%s.o" % name
    if path.exists(objfile):
        os.remove(objfile)
    analyze_cmd = "iverilog -o %s %s.v tb_%s.v" % (objfile, name, name)
    os.system(analyze_cmd)


def verilogCompileCver(name):
    cmd = "cver -c %s.v" % name
    os.system(cmd)


setupCosimulation = setupCosimulationIcarus
# setupCosimulation = setupCosimulationCver

verilogCompile = verilogCompileIcarus
# verilogCompile = verilogCompileCver



''' Bitonic sort '''

# http://www.myhdl.org/examples/bitonic/

import unittest
from random import randrange

from myhdl import Signal, intbv, \
  always_comb, instance, \
  delay, block, StopSimulation

ASCENDING = True
DESCENDING = False

# modules


@block
def compare(a_1, a_2, z_1, z_2, direction):
    """ Combinatorial circuit with two input and two output signals.
        Sorting to 'direction'. """

    @always_comb
    def comb():
        ''' Combinatorial comb '''
        if direction == (a_1 > a_2):
            z_1.next = a_2
            z_2.next = a_1
        else:
            z_1.next = a_1
            z_2.next = a_2

    return comb


@block
def feedthru(in_a, out_z):
    """ Equivalent of 'doing nothing'. """

    @always_comb
    def comb():
        ''' Combinatorial comb '''
        out_z.next = in_a

    return comb


@block
def bitonic_merge(list_a, list_z, direction):
    """ bitonicMerge:
        Generates the output from the input list of signals.
        Recursive. """
    len_list = len(list_a)
    half_len = len_list // 2
    width = len(list_a[0])

    if len_list > 1:
        tmp = [Signal(intbv(0)[width:]) for _ in range(len_list)]

        comp = [compare(list_a[i], list_a[i + half_len], tmp[i], tmp[i + half_len], \
                        direction) for i in range(half_len)]

        lo_merge = bitonic_merge(tmp[:half_len], list_z[:half_len], direction)
        hi_merge = bitonic_merge(tmp[half_len:], list_z[half_len:], direction)

        return comp, lo_merge, hi_merge
    else:
        feed = feedthru(list_a[0], list_z[0])
        return feed


@block
def bitonic_sort(list_a, list_z, direction):
    """ bitonicSort:
        Produces a bitonic sequence.
        Recursive. """
    len_list = len(list_a)
    half_len = len_list // 2
    width = len(list_a[0])

    if len_list > 1:
        tmp = [Signal(intbv(0)[width:]) for _ in range(len_list)]

        lo_sort = bitonic_sort(list_a[:half_len], tmp[:half_len], ASCENDING)
        hi_sort = bitonic_sort(list_a[half_len:], tmp[half_len:], DESCENDING)

        merge = bitonic_merge(tmp, list_z, direction)
        return lo_sort, hi_sort, merge
    else:
        feed = feedthru(list_a[0], list_z[0])
        return feed

# tests


@block
def array8sorter(a_0, a_1, a_2, a_3, a_4, a_5, a_6, a_7,
                 z_0, z_1, z_2, z_3, z_4, z_5, z_6, z_7):
    ''' Sort Array with 8 values '''

    list_a = [a_0, a_1, a_2, a_3, a_4, a_5, a_6, a_7]
    list_z = [z_0, z_1, z_2, z_3, z_4, z_5, z_6, z_7]

    sort = bitonic_sort(list_a, list_z, ASCENDING)
    return sort


class TestBitonicSort(unittest.TestCase):
    ''' Test class for bitonic sort '''

    def test_sort(self):
        """ Check the functionality of the bitonic sort """
        length = 8
        width = 4

        @block
        def test_impl():
            ''' test implementation '''
            inputs = [ Signal(intbv(0)[width:]) for _ in range(length) ]
            outputs = [ Signal(intbv(0)[width:]) for _ in range(length) ]
            z_0, z_1, z_2, z_3, z_4, z_5, z_6, z_7 = outputs
            a_0, a_1, a_2, a_3, a_4, a_5, a_6, a_7 = inputs

            inst = array8sorter(a_0, a_1, a_2, a_3, a_4, a_5, a_6, a_7,
                                z_0, z_1, z_2, z_3, z_4, z_5, z_6, z_7)

            @instance
            def check():
                ''' testbench input and validation '''
                for i in range(100):
                    data = [randrange(2 ** width) for i in range(length)]
                    for i in range(length):
                        inputs[i].next = data[i]
                    yield delay(10)
                    data.sort()
                    self.assertEqual(data, outputs, 'wrong data')
                raise StopSimulation

            return inst, check

        inst = test_impl()
        inst.run_sim()

# convert


def test_issue_167():
    ''' Convert to VHDL '''
    length = 8
    width = 4
    sigs = [Signal(intbv(0)[width:]) for _ in range(2 * length)]

    inst = array8sorter(*sigs)
    inst.convert(hdl='VHDL')


if __name__ == '__main__':
    test_issue_167()



from myhdl import block, always_comb


@block
def bin2gray(B, G):
    # INCORRECT IMPLEMENTATION
    """ Gray encoder.

    B -- binary input 
    G -- Gray encoded output
    """

    @always_comb
    def comb():
        G.next = B[0]

    return comb
